@@ -36,8 +36,10 @@ GmpvPlaylistEntry *gmpv_playlist_entry_new(	const gchar *filename,
 {
 	GmpvPlaylistEntry *entry = g_malloc(sizeof(GmpvPlaylistEntry));
 
-	entry->filename = g_strdup(filename);
-	entry->title = g_strdup(title);
+	entry->filename =	g_strdup(filename);
+	entry->title =		g_strdup(title);
+	entry->metadata =	g_ptr_array_new_with_free_func
+				((GDestroyNotify)gmpv_metadata_entry_free);
 
 	return entry;
 }
@@ -46,6 +48,7 @@ void gmpv_playlist_entry_free(GmpvPlaylistEntry *entry)
 {
 	g_free(entry->filename);
 	g_free(entry->title);
+	g_ptr_array_free(entry->metadata, TRUE);
 	g_free(entry);
 }
 
@@ -103,6 +106,14 @@ gchar *get_scripts_dir_path(void)
 					NULL );
 }
 
+gchar *get_watch_dir_path(void)
+{
+	return g_build_filename(	g_get_user_config_dir(),
+					CONFIG_DIR,
+					"watch_later",
+					NULL );
+}
+
 gchar *get_path_from_uri(const gchar *uri)
 {
 	GFile *file = g_vfs_get_file_for_uri(g_vfs_get_default(), uri);
@@ -183,3 +194,37 @@ gchar *strnjoinv(const gchar *separator, const gchar **str_array, gsize count)
 	return result;
 }
 
+void activate_action_string(GActionMap *map, const gchar *str)
+{
+	GAction *action = NULL;
+	gchar *name = NULL;
+	GVariant *param = NULL;
+	gboolean param_match = FALSE;
+
+	g_action_parse_detailed_name(str, &name, &param, NULL);
+
+	if(name)
+	{
+		const GVariantType *action_ptype;
+		const GVariantType *given_ptype;
+
+		action = g_action_map_lookup_action(map, name);
+		action_ptype = g_action_get_parameter_type(action);
+		given_ptype = param?g_variant_get_type(param):NULL;
+
+		param_match =	(action_ptype == given_ptype) ||
+				(	given_ptype &&
+					g_variant_type_is_subtype_of
+					(action_ptype, given_ptype) );
+	}
+
+	if(action && param_match)
+	{
+		g_debug("Activating action %s", str);
+		g_action_activate(action, param);
+	}
+	else
+	{
+		g_warning("Failed to activate action \"%s\"", str);
+	}
+}
