@@ -27,6 +27,12 @@
 enum
 {
 	PROP_0,
+	PROP_CHAPTERS_ENABLED,
+	PROP_DURATION,
+	PROP_ENABLED,
+	PROP_PAUSE,
+	PROP_SHOW_FULLSCREEN_BUTTON,
+	PROP_TIME_POSITION,
 	PROP_VOLUME,
 	N_PROPERTIES
 };
@@ -43,6 +49,12 @@ struct _GmpvControlBox
 	GtkWidget *volume_button;
 	GtkWidget *fullscreen_button;
 	GtkWidget *seek_bar;
+	gboolean chapters_enabled;
+	gdouble duration;
+	gboolean enabled;
+	gboolean pause;
+	gboolean show_fullscreen_button;
+	gdouble time_position;
 	gdouble volume;
 };
 
@@ -74,6 +86,9 @@ static void volume_changed_handler(	GtkVolumeButton *button,
 					gdouble value,
 					gpointer data );
 static void simple_signal_handler(GtkWidget *widget, gpointer data);
+static void set_enabled(GmpvControlBox *box, gboolean enabled);
+static void set_chapter_enabled(GmpvControlBox *box, gboolean enabled);
+static void set_playing_state(GmpvControlBox *box, gboolean playing);
 static void init_button(	GtkWidget *button,
 				const gchar *icon_name,
 				const gchar *tooltip_text );
@@ -87,13 +102,48 @@ static void set_property(	GObject *object,
 {
 	GmpvControlBox *self = GMPV_CONTROL_BOX(object);
 
-	if(property_id == PROP_VOLUME)
+	switch(property_id)
 	{
+		case PROP_CHAPTERS_ENABLED:
+		self->chapters_enabled = g_value_get_boolean(value);
+		set_chapter_enabled(self, self->chapters_enabled);
+		break;
+
+		case PROP_DURATION:
+		self->duration = g_value_get_double(value);
+
+		gmpv_seek_bar_set_duration
+			(GMPV_SEEK_BAR(self->seek_bar), self->duration);
+		break;
+
+		case PROP_ENABLED:
+		self->enabled = g_value_get_boolean(value);
+		set_enabled(self, self->enabled);
+		break;
+
+		case PROP_PAUSE:
+		self->pause = g_value_get_boolean(value);
+		set_playing_state(self, !self->pause);
+		break;
+
+		case PROP_SHOW_FULLSCREEN_BUTTON:
+		self->show_fullscreen_button = g_value_get_boolean(value);
+		break;
+
+		case PROP_TIME_POSITION:
+		self->time_position = g_value_get_double(value);
+
+		gmpv_seek_bar_set_pos
+			(GMPV_SEEK_BAR(self->seek_bar), self->time_position);
+		break;
+
+		case PROP_VOLUME:
 		self->volume = g_value_get_double(value);
-	}
-	else
-	{
+		break;
+
+		default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
+		break;
 	}
 }
 
@@ -104,13 +154,39 @@ static void get_property(	GObject *object,
 {
 	GmpvControlBox *self = GMPV_CONTROL_BOX(object);
 
-	if(property_id == PROP_VOLUME)
+	switch(property_id)
 	{
+		case PROP_CHAPTERS_ENABLED:
+		g_value_set_boolean(value, self->chapters_enabled);
+		break;
+
+		case PROP_DURATION:
+		g_value_set_double(value, self->duration);
+		break;
+
+		case PROP_ENABLED:
+		g_value_set_boolean(value, self->enabled);
+		break;
+
+		case PROP_PAUSE:
+		g_value_set_boolean(value, self->pause);
+		break;
+
+		case PROP_SHOW_FULLSCREEN_BUTTON:
+		g_value_set_boolean(value, self->show_fullscreen_button);
+		break;
+
+		case PROP_TIME_POSITION:
+		g_value_set_double(value, self->time_position);
+		break;
+
+		case PROP_VOLUME:
 		g_value_set_double(value, self->volume);
-	}
-	else
-	{
+		break;
+
+		default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
+		break;
 	}
 }
 
@@ -159,25 +235,65 @@ static void simple_signal_handler(GtkWidget *widget, gpointer data)
 	{
 		const GtkWidget *widget;
 		const gchar *name;
+		const gchar *button;
 	}
 	signal_map[]
-		= {	{box->play_button, "play-button-clicked"},
-			{box->stop_button, "stop-button-clicked"},
-			{box->forward_button, "forward-button-clicked"},
-			{box->rewind_button, "rewind-button-clicked"},
-			{box->previous_button, "previous-button-clicked"},
-			{box->next_button, "next-button-clicked"},
-			{box->fullscreen_button, "fullscreen-button-clicked"},
-			{NULL, NULL} };
+		= {	{box->play_button, "button-clicked", "play"},
+			{box->stop_button, "button-clicked", "stop"},
+			{box->forward_button, "button-clicked", "forward"},
+			{box->rewind_button, "button-clicked", "rewind"},
+			{box->previous_button, "button-clicked", "previous"},
+			{box->next_button, "button-clicked", "next"},
+			{box->fullscreen_button, "button-clicked", "fullscreen"},
+			{NULL, NULL, NULL} };
 
 	for(i = 0; signal_map[i].name && signal_map[i].widget != widget; i++);
 
 	if(signal_map[i].name)
 	{
-		g_signal_emit_by_name(data, signal_map[i].name);
+		g_signal_emit_by_name
+			(data, signal_map[i].name, signal_map[i].button);
 	}
 }
 
+static void set_enabled(GmpvControlBox *box, gboolean enabled)
+{
+	gtk_widget_set_sensitive(box->previous_button, enabled);
+	gtk_widget_set_sensitive(box->rewind_button, enabled);
+	gtk_widget_set_sensitive(box->stop_button, enabled);
+	gtk_widget_set_sensitive(box->play_button, enabled);
+	gtk_widget_set_sensitive(box->forward_button, enabled);
+	gtk_widget_set_sensitive(box->next_button, enabled);
+}
+
+static void set_chapter_enabled(GmpvControlBox *box, gboolean enabled)
+{
+	if(enabled)
+	{
+		gtk_widget_show_all(box->previous_button);
+		gtk_widget_show_all(box->next_button);
+	}
+	else
+	{
+		gtk_widget_hide(box->previous_button);
+		gtk_widget_hide(box->next_button);
+	}
+}
+
+static void set_playing_state(GmpvControlBox *box, gboolean playing)
+{
+	GtkWidget *image = gtk_button_get_image(GTK_BUTTON(box->play_button));
+	const gchar *tooltip = playing?_("Pause"):_("Play");
+
+	gtk_image_set_from_icon_name( 	GTK_IMAGE(image),
+					playing?
+					"media-playback-pause-symbolic":
+					"media-playback-start-symbolic",
+					GTK_ICON_SIZE_BUTTON );
+
+	gtk_widget_set_tooltip_text(box->play_button, tooltip);
+}
+
 static void init_button(	GtkWidget *button,
 				const gchar *icon_name,
 				const gchar *tooltip_text )
@@ -193,21 +309,64 @@ static void init_button(	GtkWidget *button,
 
 static void gmpv_control_box_class_init(GmpvControlBoxClass *klass)
 {
-	/* Names of signals that have no parameter and return nothing */
-	const gchar *simple_signals[] = {	"play-button-clicked",
-						"stop-button-clicked",
-						"forward-button-clicked",
-						"rewind-button-clicked",
-						"previous-button-clicked",
-						"next-button-clicked",
-						"fullscreen-button-clicked",
-						NULL };
 	GObjectClass *object_class = G_OBJECT_CLASS(klass);
 	GParamSpec *pspec = NULL;
 
 	object_class->set_property = set_property;
 	object_class->get_property = get_property;
 
+	pspec = g_param_spec_boolean
+		(	"chapters-enabled",
+			"Chapters enabled",
+			"Whether the chapter controls are enabled",
+			FALSE,
+			G_PARAM_READWRITE );
+	g_object_class_install_property(object_class, PROP_CHAPTERS_ENABLED, pspec);
+
+	pspec = g_param_spec_double
+		(	"duration",
+			"Duration",
+			"Duration of the file",
+			0.0,
+			G_MAXDOUBLE,
+			0.0,
+			G_PARAM_READWRITE );
+	g_object_class_install_property(object_class, PROP_DURATION, pspec);
+
+	pspec = g_param_spec_boolean
+		(	"enabled",
+			"Enabled",
+			"Whether the controls are enabled",
+			TRUE,
+			G_PARAM_READWRITE );
+	g_object_class_install_property(object_class, PROP_ENABLED, pspec);
+
+	pspec = g_param_spec_boolean
+		(	"pause",
+			"Pause",
+			"Whether there is a file playing",
+			TRUE,
+			G_PARAM_READWRITE );
+	g_object_class_install_property(object_class, PROP_PAUSE, pspec);
+
+	pspec = g_param_spec_boolean
+		(	"show-fullscreen-button",
+			"Show fullscreen button",
+			"Whether to show fullscreen button when applicable",
+			FALSE,
+			G_PARAM_READWRITE );
+	g_object_class_install_property(object_class, PROP_SHOW_FULLSCREEN_BUTTON, pspec);
+
+	pspec = g_param_spec_double
+		(	"time-position",
+			"Time position",
+			"The current timr position in the current file",
+			0.0,
+			G_MAXDOUBLE,
+			0.0,
+			G_PARAM_READWRITE );
+	g_object_class_install_property(object_class, PROP_TIME_POSITION, pspec);
+
 	pspec = g_param_spec_double
 		(	"volume",
 			"Volume",
@@ -218,19 +377,16 @@ static void gmpv_control_box_class_init(GmpvControlBoxClass *klass)
 			G_PARAM_READWRITE );
 	g_object_class_install_property(object_class, PROP_VOLUME, pspec);
 
-	for(gint i = 0; simple_signals[i]; i++)
-	{
-		g_signal_new(	simple_signals[i],
-				G_TYPE_FROM_CLASS(klass),
-				G_SIGNAL_RUN_FIRST,
-				0,
-				NULL,
-				NULL,
-				g_cclosure_marshal_VOID__VOID,
-				G_TYPE_NONE,
-				0 );
-	}
-
+	g_signal_new(	"button-clicked",
+			G_TYPE_FROM_CLASS(klass),
+			G_SIGNAL_RUN_FIRST,
+			0,
+			NULL,
+			NULL,
+			g_cclosure_marshal_VOID__STRING,
+			G_TYPE_NONE,
+			1,
+			G_TYPE_STRING );
 	g_signal_new(	"seek",
 			G_TYPE_FROM_CLASS(klass),
 			G_SIGNAL_RUN_FIRST,
@@ -241,7 +397,6 @@ static void gmpv_control_box_class_init(GmpvControlBoxClass *klass)
 			G_TYPE_NONE,
 			1,
 			G_TYPE_DOUBLE );
-
 	g_signal_new(	"volume-changed",
 			G_TYPE_FROM_CLASS(klass),
 			G_SIGNAL_RUN_FIRST,
@@ -265,6 +420,13 @@ static void gmpv_control_box_init(GmpvControlBox *box)
 	box->fullscreen_button = gtk_button_new();
 	box->volume_button = gtk_volume_button_new();
 	box->seek_bar = gmpv_seek_bar_new();
+	box->chapters_enabled = FALSE;
+	box->duration = 0.0;
+	box->enabled = TRUE;
+	box->pause = TRUE;
+	box->show_fullscreen_button = FALSE;
+	box->time_position = 0.0;
+	box->volume = 0.0;
 
 	init_button(	box->play_button,
 			"media-playback-start-symbolic",
@@ -360,30 +522,6 @@ GtkWidget *gmpv_control_box_new(void)
 	return GTK_WIDGET(g_object_new(gmpv_control_box_get_type(), NULL));
 }
 
-void gmpv_control_box_set_enabled(GmpvControlBox *box, gboolean enabled)
-{
-	gtk_widget_set_sensitive(box->previous_button, enabled);
-	gtk_widget_set_sensitive(box->rewind_button, enabled);
-	gtk_widget_set_sensitive(box->stop_button, enabled);
-	gtk_widget_set_sensitive(box->play_button, enabled);
-	gtk_widget_set_sensitive(box->forward_button, enabled);
-	gtk_widget_set_sensitive(box->next_button, enabled);
-}
-
-void gmpv_control_box_set_chapter_enabled(GmpvControlBox *box, gboolean enabled)
-{
-	if(enabled)
-	{
-		gtk_widget_show_all(box->previous_button);
-		gtk_widget_show_all(box->next_button);
-	}
-	else
-	{
-		gtk_widget_hide(box->previous_button);
-		gtk_widget_hide(box->next_button);
-	}
-}
-
 void gmpv_control_box_set_seek_bar_pos(GmpvControlBox *box, gdouble pos)
 {
 	gmpv_seek_bar_set_pos(GMPV_SEEK_BAR(box->seek_bar), pos);
@@ -416,20 +554,6 @@ gboolean gmpv_control_box_get_volume_popup_visible(GmpvControlBox *box)
 	return gtk_widget_is_visible(gtk_scale_button_get_popup(volume_button));
 }
 
-void gmpv_control_box_set_playing_state(GmpvControlBox *box, gboolean playing)
-{
-	GtkWidget *image = gtk_button_get_image(GTK_BUTTON(box->play_button));
-	const gchar *tooltip = playing?_("Pause"):_("Play");
-
-	gtk_image_set_from_icon_name( 	GTK_IMAGE(image),
-					playing?
-					"media-playback-pause-symbolic":
-					"media-playback-start-symbolic",
-					GTK_ICON_SIZE_BUTTON );
-
-	gtk_widget_set_tooltip_text(box->play_button, tooltip);
-}
-
 void gmpv_control_box_set_fullscreen_state(	GmpvControlBox *box,
 						gboolean fullscreen )
 {
@@ -440,19 +564,16 @@ void gmpv_control_box_set_fullscreen_state(	GmpvControlBox *box,
 					"view-restore-symbolic":
 					"view-fullscreen-symbolic",
 					GTK_ICON_SIZE_BUTTON );
-}
 
-void gmpv_control_box_set_fullscreen_button_visible(	GmpvControlBox *box,
-							gboolean value )
-{
-	gtk_widget_set_visible(box->fullscreen_button, value);
+	gtk_widget_set_visible(	box->fullscreen_button,
+				!fullscreen && box->show_fullscreen_button );
 }
 
 void gmpv_control_box_reset(GmpvControlBox *box)
 {
 	gmpv_control_box_set_seek_bar_pos(box, 0);
 	gmpv_control_box_set_seek_bar_duration(box, 0);
-	gmpv_control_box_set_playing_state(box, FALSE);
-	gmpv_control_box_set_chapter_enabled(box, FALSE);
+	set_playing_state(box, FALSE);
+	set_chapter_enabled(box, FALSE);
 	gmpv_control_box_set_fullscreen_state(box, FALSE);
 }
