@@ -26,6 +26,7 @@
 
 #include "gmpv_controller_private.h"
 #include "gmpv_controller.h"
+#include "gmpv_controller_actions.h"
 #include "gmpv_controller_input.h"
 #include "gmpv_def.h"
 
@@ -38,6 +39,13 @@ static void get_property(	GObject *object,
 				guint property_id,
 				GValue *value,
 				GParamSpec *pspec );
+static void dispose(GObject *object);
+static void mpris_enable_handler(	GSettings *settings,
+					gchar *key,
+					gpointer data );
+static void media_keys_enable_handler(	GSettings *settings,
+					gchar *key,
+					gpointer data );
 static void view_ready_handler(GmpvView *view, gpointer data);
 static void render_handler(GmpvView *view, gpointer data);
 static void preferences_updated_handler(GmpvView *view, gpointer data);
@@ -65,22 +73,6 @@ static void playlist_reordered_handler(	GmpvView *view,
 					gpointer data );
 static void connect_signals(GmpvController *controller);
 static gboolean update_seek_bar(gpointer data);
-static gboolean track_str_to_int(	GBinding *binding,
-					const GValue *from_value,
-					GValue *to_value,
-					gpointer data );
-static gboolean int_to_track_str(	GBinding *binding,
-					const GValue *from_value,
-					GValue *to_value,
-					gpointer data );
-static gboolean loop_str_to_boolean(	GBinding *binding,
-					const GValue *from_value,
-					GValue *to_value,
-					gpointer data );
-static gboolean boolean_to_loop_str(	GBinding *binding,
-					const GValue *from_value,
-					GValue *to_value,
-					gpointer data );
 static gboolean is_more_than_one(	GBinding *binding,
 					const GValue *from_value,
 					GValue *to_value,
@@ -91,21 +83,37 @@ static void idle_active_handler(	GObject *object,
 static void playlist_handler(		GObject *object,
 					GParamSpec *pspec,
 					gpointer data);
+static void vid_handler(		GObject *object,
+					GParamSpec *pspec,
+					gpointer data);
 static void model_ready_handler(	GObject *object,
 					GParamSpec *pspec,
 					gpointer data );
 static void frame_ready_handler(GmpvModel *model, gpointer data);
-static void autofit_handler(GmpvModel *model, gdouble multiplier, gpointer data);
+static void metadata_update_handler(GmpvModel *model, gint64 pos, gpointer data);
+static void window_resize_handler(	GmpvModel *model,
+					gint64 width,
+					gint64 height,
+					gpointer data );
+static void window_move_handler(	GmpvModel *model,
+					gboolean flip_x,
+					gboolean flip_y,
+					GValue *x,
+					GValue *y,
+					gpointer data );
 static void message_handler(GmpvMpv *mpv, const gchar *message, gpointer data);
 static void error_handler(GmpvMpv *mpv, const gchar *message, gpointer data);
 static void shutdown_handler(GmpvMpv *mpv, gpointer data);
-static void post_shutdown_handler(GmpvMpv *mpv, gpointer data);
+static void fullscreen_handler(		GObject *object,
+					GParamSpec *pspec,
+					gpointer data );
 static void play_button_handler(GtkButton *button, gpointer data);
 static void stop_button_handler(GtkButton *button, gpointer data);
 static void forward_button_handler(GtkButton *button, gpointer data);
 static void rewind_button_handler(GtkButton *button, gpointer data);
 static void next_button_handler(GtkButton *button, gpointer data);
 static void previous_button_handler(GtkButton *button, gpointer data);
+static void fullscreen_button_handler(GtkButton *button, gpointer data);
 static void seek_handler(GtkButton *button, gdouble value, gpointer data);
 static void gmpv_controller_class_init(GmpvControllerClass *klass);
 static void gmpv_controller_init(GmpvController *controller);
@@ -114,10 +122,45 @@ G_DEFINE_TYPE(GmpvController, gmpv_controller, G_TYPE_OBJECT)
 
 static void constructed(GObject *object)
 {
-	GmpvController *controller = GMPV_CONTROLLER(object);
+	GmpvController *controller;
+	GmpvMainWindow *window;
+	gboolean always_floating;
+	gint64 wid;
+
+	controller = GMPV_CONTROLLER(object);
+	always_floating =	g_settings_get_boolean
+				(	controller->settings,
+					"always-use-floating-controls" );
+
+	controller->view = gmpv_view_new(controller->app, always_floating);
+	window = gmpv_view_get_main_window(controller->view);
+	wid = gmpv_video_area_get_xid(gmpv_main_window_get_video_area(window));
+	controller->model = gmpv_model_new(wid);
 
 	connect_signals(controller);
+	gmpv_controller_action_register_actions(controller);
 	gmpv_controller_input_connect_signals(controller);
+
+	g_signal_connect(	controller->settings,
+				"changed::mpris-enable",
+				G_CALLBACK(mpris_enable_handler),
+				controller );
+	g_signal_connect(	controller->settings,
+				"changed::media-keys-enable",
+				G_CALLBACK(media_keys_enable_handler),
+				controller );
+
+	if(g_settings_get_boolean(controller->settings, "mpris-enable"))
+	{
+		controller->mpris = gmpv_mpris_new(controller);
+	}
+
+	if(g_settings_get_boolean(controller->settings, "media-keys-enable"))
+	{
+		controller->media_keys = gmpv_media_keys_new(controller);
+	}
+
+	G_OBJECT_CLASS(gmpv_controller_parent_class)->constructed(object);
 }
 
 static void set_property(	GObject *object,
@@ -129,34 +172,14 @@ static void set_property(	GObject *object,
 
 	switch(property_id)
 	{
-		case PROP_MODEL:
-		self->model = g_value_get_pointer(value);
-		break;
-
-		case PROP_VIEW:
-		self->view = g_value_get_pointer(value);
+		case PROP_APP:
+		self->app = g_value_get_pointer(value);
 		break;
 
 		case PROP_READY:
 		self->ready = g_value_get_boolean(value);
 		break;
 
-		case PROP_AID:
-		self->aid = g_value_get_int(value);
-		break;
-
-		case PROP_VID:
-		self->vid = g_value_get_int(value);
-		break;
-
-		case PROP_SID:
-		self->sid = g_value_get_int(value);
-		break;
-
-		case PROP_LOOP:
-		self->loop = g_value_get_boolean(value);
-		break;
-
 		case PROP_IDLE:
 		self->idle = g_value_get_boolean(value);
 		break;
@@ -176,34 +199,14 @@ static void get_property(	GObject *object,
 
 	switch(property_id)
 	{
-		case PROP_MODEL:
-		g_value_set_pointer(value, self->model);
-		break;
-
-		case PROP_VIEW:
-		g_value_set_pointer(value, self->view);
+		case PROP_APP:
+		g_value_set_pointer(value, self->app);
 		break;
 
 		case PROP_READY:
 		g_value_set_boolean(value, self->ready);
 		break;
 
-		case PROP_AID:
-		g_value_set_int(value, self->aid);
-		break;
-
-		case PROP_VID:
-		g_value_set_int(value, self->vid);
-		break;
-
-		case PROP_SID:
-		g_value_set_int(value, self->sid);
-		break;
-
-		case PROP_LOOP:
-		g_value_set_boolean(value, self->loop);
-		break;
-
 		case PROP_IDLE:
 		g_value_set_boolean(value, self->idle);
 		break;
@@ -214,6 +217,62 @@ static void get_property(	GObject *object,
 	}
 }
 
+static void dispose(GObject *object)
+{
+	GmpvController *controller = GMPV_CONTROLLER(object);
+
+	g_clear_object(&controller->settings);
+	g_clear_object(&controller->mpris);
+	g_clear_object(&controller->media_keys);
+
+	if(controller->update_seekbar_id != 0)
+	{
+		g_source_remove(controller->update_seekbar_id);
+		controller->update_seekbar_id = 0;
+	}
+
+	if(controller->view)
+	{
+		gmpv_view_make_gl_context_current(controller->view);
+		g_clear_object(&controller->model);
+		g_clear_object(&controller->view);
+	}
+
+	G_OBJECT_CLASS(gmpv_controller_parent_class)->dispose(object);
+}
+
+static void mpris_enable_handler(	GSettings *settings,
+					gchar *key,
+					gpointer data )
+{
+	GmpvController *controller = data;
+
+	if(!controller->mpris && g_settings_get_boolean(settings, key))
+	{
+		controller->mpris = gmpv_mpris_new(controller);
+	}
+	else if(controller->mpris)
+	{
+		g_clear_object(&controller->mpris);
+	}
+}
+
+static void media_keys_enable_handler(	GSettings *settings,
+					gchar *key,
+					gpointer data )
+{
+	GmpvController *controller = data;
+
+	if(!controller->media_keys && g_settings_get_boolean(settings, key))
+	{
+		controller->media_keys = gmpv_media_keys_new(controller);
+	}
+	else if(controller->media_keys)
+	{
+		g_clear_object(&controller->media_keys);
+	}
+}
+
 static void view_ready_handler(GmpvView *view, gpointer data)
 {
 	gmpv_model_initialize(GMPV_CONTROLLER(data)->model);
@@ -229,11 +288,6 @@ static void render_handler(GmpvView *view, gpointer data)
 	scale = gmpv_view_get_scale_factor(controller->view);
 	gmpv_view_get_video_area_geometry(controller->view, &width, &height);
 	gmpv_model_render_frame(controller->model, scale*width, scale*height);
-
-	while(gtk_events_pending())
-	{
-		gtk_main_iteration();
-	}
 }
 
 static void preferences_updated_handler(GmpvView *view, gpointer data)
@@ -278,7 +332,6 @@ static void grab_handler(GmpvView *view, gboolean was_grabbed, gpointer data)
 
 static void delete_handler(GmpvView *view, gpointer data)
 {
-	gmpv_model_quit(GMPV_CONTROLLER(data)->model);
 	g_signal_emit_by_name(data, "shutdown");
 }
 
@@ -319,38 +372,9 @@ static void playlist_reordered_handler(	GmpvView *view,
 
 static void connect_signals(GmpvController *controller)
 {
-	g_object_bind_property_full(	controller->model, "aid",
-					controller, "aid",
-					G_BINDING_BIDIRECTIONAL,
-					track_str_to_int,
-					int_to_track_str,
-					NULL,
-					NULL );
-	g_object_bind_property_full(	controller->model, "vid",
-					controller, "vid",
-					G_BINDING_BIDIRECTIONAL,
-					track_str_to_int,
-					int_to_track_str,
-					NULL,
-					NULL );
-	g_object_bind_property_full(	controller->model, "sid",
-					controller, "sid",
-					G_BINDING_BIDIRECTIONAL,
-					track_str_to_int,
-					int_to_track_str,
-					NULL,
-					NULL );
-	g_object_bind_property_full(	controller->model, "loop",
-					controller, "loop",
-					G_BINDING_BIDIRECTIONAL,
-					loop_str_to_boolean,
-					boolean_to_loop_str,
-					NULL,
-					NULL );
 	g_object_bind_property(	controller->model, "core-idle",
 				controller, "idle",
 				G_BINDING_DEFAULT );
-
 	g_object_bind_property(	controller->model, "pause",
 				controller->view, "pause",
 				G_BINDING_DEFAULT );
@@ -389,13 +413,25 @@ static void connect_signals(GmpvController *controller)
 				"notify::playlist",
 				G_CALLBACK(playlist_handler),
 				controller );
+	g_signal_connect(	controller->model,
+				"notify::vid",
+				G_CALLBACK(vid_handler),
+				controller );
 	g_signal_connect(	controller->model,
 				"frame-ready",
 				G_CALLBACK(frame_ready_handler),
 				controller );
 	g_signal_connect(	controller->model,
-				"autofit",
-				G_CALLBACK(autofit_handler),
+				"metadata-update",
+				G_CALLBACK(metadata_update_handler),
+				controller );
+	g_signal_connect(	controller->model,
+				"window-resize",
+				G_CALLBACK(window_resize_handler),
+				controller );
+	g_signal_connect(	controller->model,
+				"window-move",
+				G_CALLBACK(window_move_handler),
 				controller );
 	g_signal_connect(	controller->model,
 				"message",
@@ -409,11 +445,11 @@ static void connect_signals(GmpvController *controller)
 				"shutdown",
 				G_CALLBACK(shutdown_handler),
 				controller );
-	g_signal_connect_after(	controller->model,
-				"shutdown",
-				G_CALLBACK(post_shutdown_handler),
-				controller );
 
+	g_signal_connect(	controller->view,
+				"notify::fullscreen",
+				G_CALLBACK(fullscreen_handler),
+				controller );
 	g_signal_connect(	controller->view,
 				"button-clicked::play",
 				G_CALLBACK(play_button_handler),
@@ -438,6 +474,10 @@ static void connect_signals(GmpvController *controller)
 				"button-clicked::previous",
 				G_CALLBACK(previous_button_handler),
 				controller );
+	g_signal_connect(	controller->view,
+				"button-clicked::fullscreen",
+				G_CALLBACK(fullscreen_button_handler),
+				controller );
 	g_signal_connect(	controller->view,
 				"seek",
 				G_CALLBACK(seek_handler),
@@ -504,63 +544,6 @@ gboolean update_seek_bar(gpointer data)
 	return TRUE;
 }
 
-static gboolean track_str_to_int(	GBinding *binding,
-					const GValue *from_value,
-					GValue *to_value,
-					gpointer data )
-{
-	gchar *endptr = NULL;
-	const gchar *from = g_value_get_string(from_value);
-	gint to = (gint)g_ascii_strtoll(from, &endptr, 10);
-
-	if(to == 0 && g_strcmp0(from, "no") != 0)
-	{
-		to = -1;
-	}
-
-	g_value_set_int(to_value, to);
-
-	return TRUE;
-}
-
-static gboolean int_to_track_str(	GBinding *binding,
-					const GValue *from_value,
-					GValue *to_value,
-					gpointer data )
-{
-	gint from = g_value_get_int(from_value);
-	gchar buf[16] = "no";
-
-	g_snprintf(buf, 16, "%d", from);
-	g_value_set_string(to_value, buf);
-
-	return TRUE;
-}
-
-static gboolean loop_str_to_boolean(	GBinding *binding,
-					const GValue *from_value,
-					GValue *to_value,
-					gpointer data )
-{
-	const gchar *from = g_value_get_string(from_value);
-
-	g_value_set_boolean(to_value, g_strcmp0(from, "no") != 0);
-
-	return TRUE;
-}
-
-static gboolean boolean_to_loop_str(	GBinding *binding,
-					const GValue *from_value,
-					GValue *to_value,
-					gpointer data )
-{
-	gboolean from = g_value_get_boolean(from_value);
-
-	g_value_set_string(to_value, from?"inf":"no");
-
-	return TRUE;
-}
-
 static gboolean is_more_than_one(	GBinding *binding,
 					const GValue *from_value,
 					GValue *to_value,
@@ -601,11 +584,33 @@ static void playlist_handler(	GObject *object,
 	GPtrArray *playlist = NULL;
 	gint64 pos = 0;
 
-	g_object_get(object, "playlist", &playlist, "playlist-pos", &pos, NULL);
+	g_object_get(	object,
+			"playlist", &playlist,
+			"playlist-pos", &pos,
+			NULL );
+
 	gmpv_view_update_playlist(view, playlist);
 	gmpv_view_set_playlist_pos(view, pos);
 }
 
+static void vid_handler(	GObject *object,
+				GParamSpec *pspec,
+				gpointer data )
+{
+	GmpvController *controller = data;
+	GmpvMainWindow *window = gmpv_view_get_main_window(controller->view);
+	GActionMap *map = G_ACTION_MAP(window);
+	GAction *action = g_action_map_lookup_action(map, "set-video-size");
+	gchar *vid_str = NULL;
+	gint64 vid = 0;
+
+	g_object_get(object, "vid", &vid_str, NULL);
+	vid = g_ascii_strtoll(vid_str, NULL, 10);
+	g_simple_action_set_enabled(G_SIMPLE_ACTION(action), vid > 0);
+
+	g_free(vid_str);
+}
+
 static void model_ready_handler(	GObject *object,
 					GParamSpec *pspec,
 					gpointer data )
@@ -639,33 +644,68 @@ static void frame_ready_handler(GmpvModel *model, gpointer data)
 	gmpv_view_queue_render(GMPV_CONTROLLER(data)->view);
 }
 
-static void autofit_handler(GmpvModel *model, gdouble multiplier, gpointer data)
+static void metadata_update_handler(GmpvModel *model, gint64 pos, gpointer data)
+{
+	GmpvView *view = GMPV_CONTROLLER(data)->view;
+	GPtrArray *playlist = NULL;
+
+	g_object_get(G_OBJECT(model), "playlist", &playlist, NULL);
+	gmpv_view_update_playlist(view, playlist);
+}
+
+static void window_resize_handler(	GmpvModel *model,
+					gint64 width,
+					gint64 height,
+					gpointer data )
 {
 	GmpvController *controller = data;
-	gint64 width = -1;
-	gint64 height = -1;
 
-	gmpv_model_get_video_geometry(model, &width, &height);
-
-	if(width > 0 && height > 0)
-	{
-		gint new_width;
-		gint new_height;
-
-		new_width = (gint)(multiplier*(gdouble)width);
-		new_height = (gint)(multiplier*(gdouble)height);
-
-		g_debug("Resizing window to %dx%d", new_width, new_height);
-
-		gmpv_view_resize_video_area
-			(controller->view, new_width, new_height);
-	}
+	gmpv_view_resize_video_area(controller->view, (gint)width, (gint)height);
+}
 
+static void window_move_handler(	GmpvModel *model,
+					gboolean flip_x,
+					gboolean flip_y,
+					GValue *x,
+					GValue *y,
+					gpointer data )
+{
+	gmpv_view_move(GMPV_CONTROLLER(data)->view, flip_x, flip_y, x, y);
 }
 
 static void message_handler(GmpvMpv *mpv, const gchar *message, gpointer data)
 {
-	g_signal_emit_by_name(data, "message", message);
+	const gsize prefix_length = sizeof(ACTION_PREFIX)-1;
+
+	/* Verify that both the prefix and the scope matches */
+	if(message && strncmp(message, ACTION_PREFIX, prefix_length) == 0)
+	{
+		const gchar *action = message+prefix_length+1;
+		GmpvController *controller = data;
+		GmpvView *view = controller->view;
+		GActionMap *map = NULL;
+
+		if(g_str_has_prefix(action, "win."))
+		{
+			map = G_ACTION_MAP(gmpv_view_get_main_window(view));
+		}
+		else if(g_str_has_prefix(action, "app."))
+		{
+			map = G_ACTION_MAP(controller->app);
+		}
+
+		if(map)
+		{
+			/* Strip scope and activate */
+			activate_action_string(map, strchr(action, '.')+1);
+		}
+		else
+		{
+			g_warning(	"Received action with missing or "
+					"unknown scope %s",
+					action );
+		}
+	}
 }
 
 static void error_handler(GmpvMpv *mpv, const gchar *message, gpointer data)
@@ -679,15 +719,29 @@ static void error_handler(GmpvMpv *mpv, const gchar *message, gpointer data)
 
 static void shutdown_handler(GmpvMpv *mpv, gpointer data)
 {
-	GmpvController *controller = data;
-
-	g_source_remove(controller->update_seekbar_id);
-	gmpv_view_make_gl_context_current(controller->view);
+	g_signal_emit_by_name(data, "shutdown");
 }
 
-static void post_shutdown_handler(GmpvMpv *mpv, gpointer data)
+static void fullscreen_handler(		GObject *object,
+					GParamSpec *pspec,
+					gpointer data )
 {
-	gmpv_view_quit(GMPV_CONTROLLER(data)->view);
+	GmpvView *view = GMPV_CONTROLLER(data)->view;
+	GmpvMainWindow *window = gmpv_view_get_main_window(view);
+	GActionMap *map = G_ACTION_MAP(window);
+	GAction *toggle_playlist = NULL;
+	GAction *toggle_controls = NULL;
+	gboolean fullscreen = FALSE;
+
+	toggle_playlist = g_action_map_lookup_action(map, "toggle-playlist");
+	toggle_controls = g_action_map_lookup_action(map, "toggle-controls");
+
+	g_object_get(view, "fullscreen", &fullscreen, NULL);
+
+	g_simple_action_set_enabled
+		(G_SIMPLE_ACTION(toggle_playlist), !fullscreen);
+	g_simple_action_set_enabled
+		(G_SIMPLE_ACTION(toggle_controls), !fullscreen);
 }
 
 static void play_button_handler(GtkButton *button, gpointer data)
@@ -732,6 +786,15 @@ static void previous_button_handler(GtkButton *button, gpointer data)
 	gmpv_model_previous_chapter(GMPV_CONTROLLER(data)->model);
 }
 
+static void fullscreen_button_handler(GtkButton *button, gpointer data)
+{
+	GmpvView *view = GMPV_CONTROLLER(data)->view;
+	gboolean fullscreen = FALSE;
+
+	g_object_get(view, "fullscreen", &fullscreen, NULL);
+	g_object_set(view, "fullscreen", !fullscreen, NULL);
+}
+
 static void seek_handler(GtkButton *button, gdouble value, gpointer data)
 {
 	gmpv_model_seek(GMPV_CONTROLLER(data)->model, value);
@@ -745,50 +808,14 @@ static void gmpv_controller_class_init(GmpvControllerClass *klass)
 	obj_class->constructed = constructed;
 	obj_class->set_property = set_property;
 	obj_class->get_property = get_property;
+	obj_class->dispose = dispose;
 
 	pspec = g_param_spec_pointer
-		(	"model",
-			"Model",
-			"The GmpvModel to use",
+		(	"app",
+			"App",
+			"The GmpvApplication to use",
 			G_PARAM_CONSTRUCT_ONLY|G_PARAM_READWRITE );
-	g_object_class_install_property(obj_class, PROP_MODEL, pspec);
-
-	pspec = g_param_spec_pointer
-		(	"view",
-			"View",
-			"The GmpvView to use",
-			G_PARAM_CONSTRUCT_ONLY|G_PARAM_READWRITE );
-	g_object_class_install_property(obj_class, PROP_VIEW, pspec);
-
-	pspec = g_param_spec_int
-		(	"aid",
-			"Audio track ID",
-			"The ID of the current audio track",
-			-1,
-			G_MAXINT,
-			-1,
-			G_PARAM_READWRITE );
-	g_object_class_install_property(obj_class, PROP_AID, pspec);
-
-	pspec = g_param_spec_int
-		(	"vid",
-			"Video track ID",
-			"The ID of the current video track",
-			-1,
-			G_MAXINT,
-			-1,
-			G_PARAM_READWRITE );
-	g_object_class_install_property(obj_class, PROP_VID, pspec);
-
-	pspec = g_param_spec_int
-		(	"sid",
-			"Subtitle track ID",
-			"The ID of the current subtitle track",
-			-1,
-			G_MAXINT,
-			-1,
-			G_PARAM_READWRITE );
-	g_object_class_install_property(obj_class, PROP_SID, pspec);
+	g_object_class_install_property(obj_class, PROP_APP, pspec);
 
 	pspec = g_param_spec_boolean
 		(	"ready",
@@ -798,14 +825,6 @@ static void gmpv_controller_class_init(GmpvControllerClass *klass)
 			G_PARAM_READWRITE );
 	g_object_class_install_property(obj_class, PROP_READY, pspec);
 
-	pspec = g_param_spec_boolean
-		(	"loop",
-			"Loop",
-			"Whether or not to loop when the playlist ends",
-			FALSE,
-			G_PARAM_READWRITE );
-	g_object_class_install_property(obj_class, PROP_LOOP, pspec);
-
 	pspec = g_param_spec_boolean
 		(	"idle",
 			"Idle",
@@ -814,16 +833,6 @@ static void gmpv_controller_class_init(GmpvControllerClass *klass)
 			G_PARAM_READWRITE );
 	g_object_class_install_property(obj_class, PROP_IDLE, pspec);
 
-	g_signal_new(	"message",
-			G_TYPE_FROM_CLASS(klass),
-			G_SIGNAL_RUN_FIRST,
-			0,
-			NULL,
-			NULL,
-			g_cclosure_marshal_VOID__STRING,
-			G_TYPE_NONE,
-			1,
-			G_TYPE_STRING );
 	g_signal_new(	"shutdown",
 			G_TYPE_FROM_CLASS(klass),
 			G_SIGNAL_RUN_FIRST,
@@ -837,26 +846,22 @@ static void gmpv_controller_class_init(GmpvControllerClass *klass)
 
 static void gmpv_controller_init(GmpvController *controller)
 {
+	controller->app = NULL;
 	controller->model = NULL;
 	controller->view = NULL;
-	controller->aid = 0;
-	controller->vid = 0;
-	controller->sid = 0;
 	controller->ready = FALSE;
-	controller->loop = FALSE;
 	controller->idle = TRUE;
-	controller->action_queue = g_queue_new();
-	controller->files = NULL;
-	controller->inhibit_cookie = 0;
 	controller->target_playlist_pos = -1;
 	controller->update_seekbar_id = 0;
+	controller->settings = g_settings_new(CONFIG_ROOT);
+	controller->media_keys = NULL;
+	controller->mpris = NULL;
 }
 
-GmpvController *gmpv_controller_new(GmpvModel *model, GmpvView *view)
+GmpvController *gmpv_controller_new(GmpvApplication *app)
 {
 	return GMPV_CONTROLLER(g_object_new(	gmpv_controller_get_type(),
-						"model", model,
-						"view", view,
+						"app", app,
 						NULL ));
 }
 
@@ -899,3 +904,14 @@ void gmpv_controller_open(	GmpvController *controller,
 {
 	gmpv_model_load_file(controller->model, uri, append);
 }
+
+GmpvView *gmpv_controller_get_view(GmpvController *controller)
+{
+	return controller->view;
+}
+
+GmpvModel *gmpv_controller_get_model(GmpvController *controller)
+{
+	return controller->model;
+}
+
