@@ -31,13 +31,15 @@ G_BEGIN_DECLS
 
 G_DECLARE_FINAL_TYPE(GmpvController, gmpv_controller, GMPV, CONTROLLER, GObject)
 
-GmpvController *gmpv_controller_new(GmpvModel *model, GmpvView *view);
+GmpvController *gmpv_controller_new(GmpvApplication *app);
 void gmpv_controller_quit(GmpvController *controller);
 void gmpv_controller_autofit(GmpvController *controller, gdouble multiplier);
 void gmpv_controller_present(GmpvController *controller);
 void gmpv_controller_open(	GmpvController *controller,
 				const gchar *urii,
 				gboolean append );
+GmpvView *gmpv_controller_get_view(GmpvController *controller);
+GmpvModel *gmpv_controller_get_model(GmpvController *controller);
 
 G_END_DECLS
 
