@@ -22,19 +22,16 @@
 
 #include "gmpv_model.h"
 #include "gmpv_view.h"
+#include "mpris/gmpv_mpris.h"
+#include "media_keys/gmpv_media_keys.h"
 
 G_BEGIN_DECLS
 
 enum
 {
 	PROP_0,
-	PROP_MODEL,
-	PROP_VIEW,
-	PROP_AID,
-	PROP_VID,
-	PROP_SID,
+	PROP_APP,
 	PROP_READY,
-	PROP_LOOP,
 	PROP_IDLE,
 	N_PROPERTIES
 };
@@ -42,19 +39,16 @@ enum
 struct _GmpvController
 {
 	GObject parent;
+	GmpvApplication *app;
 	GmpvModel *model;
 	GmpvView *view;
-	gint aid;
-	gint vid;
-	gint sid;
 	gboolean ready;
-	gboolean loop;
 	gboolean idle;
-	GQueue *action_queue;
-	gchar **files;
-	guint inhibit_cookie;
 	gint64 target_playlist_pos;
 	guint update_seekbar_id;
+	GSettings *settings;
+	GmpvMediaKeys *media_keys;
+	GmpvMpris *mpris;
 };
 
 struct _GmpvControllerClass
