@@ -54,6 +54,7 @@ struct _GmpvMainWindow
 	gboolean pre_fs_playlist_visible;
 	gint playlist_width;
 	guint timeout_tag;
+	const GPtrArray *track_list;
 	GtkWidget *header_bar;
 	GtkWidget *main_box;
 	GtkWidget *vid_area_paned;
@@ -76,12 +77,15 @@ static void gmpv_main_window_get_property(	GObject *object,
 						guint property_id,
 						GValue *value,
 						GParamSpec *pspec );
+static void seek_handler(GtkWidget *widget, gdouble value, gpointer data);
+static void button_clicked_handler(	GmpvControlBox *control_box,
+					const gchar *button,
+					gpointer data );
 static void gmpv_main_window_notify(GObject *object, GParamSpec *pspec);
 static void resize_video_area_finalize(	GtkWidget *widget,
 					GdkRectangle *allocation,
 					gpointer data );
 static gboolean resize_to_target(gpointer data);
-static void set_window_pos(GmpvMainWindow *wnd, const GmpvGeometry *geom);
 
 G_DEFINE_TYPE(GmpvMainWindow, gmpv_main_window, GTK_TYPE_APPLICATION_WINDOW)
 
@@ -145,6 +149,18 @@ static void gmpv_main_window_get_property(	GObject *object,
 	}
 }
 
+static void seek_handler(GtkWidget *widget, gdouble value, gpointer data)
+{
+	g_signal_emit_by_name(data, "seek", value);
+}
+
+static void button_clicked_handler(	GmpvControlBox *control_box,
+					const gchar *button,
+					gpointer data )
+{
+	g_signal_emit_by_name(data, "button-clicked", button);
+}
+
 static void gmpv_main_window_notify(GObject *object, GParamSpec *pspec)
 {
 	if(g_strcmp0(pspec->name, "always-use-floating-controls") == 0)
@@ -181,7 +197,7 @@ static void resize_video_area_finalize(	GtkWidget *widget,
 		wnd->width_offset += target_width-width;
 		wnd->height_offset += target_height-height;
 
-		g_idle_add(resize_to_target, wnd);
+		g_idle_add_full(G_PRIORITY_HIGH_IDLE, resize_to_target, wnd, NULL);
 	}
 }
 
@@ -204,69 +220,6 @@ static gboolean resize_to_target(gpointer data)
 	return FALSE;
 }
 
-static void set_window_pos(GmpvMainWindow *wnd, const GmpvGeometry *geom)
-{
-	GmpvControlBox *box;
-	GdkScreen *screen;
-	gint screen_dim[2];
-	gint window_dim[2];
-	gboolean flip[2];
-	GValue geom_pos[2];
-	gint64 pos[2];
-
-	g_assert(geom);
-
-	box = gmpv_main_window_get_control_box(wnd);
-	screen = gdk_screen_get_default();
-	screen_dim[0] = gdk_screen_get_width(screen);
-	screen_dim[1] = gdk_screen_get_height(screen);
-	window_dim[0] = 0;
-	window_dim[1] = 0;
-	geom_pos[0] = geom->x;
-	geom_pos[1] = geom->y;
-	flip[0] = geom->flags&GMPV_GEOMETRY_FLIP_X;
-	flip[1] = geom->flags&GMPV_GEOMETRY_FLIP_Y;
-	pos[0] = 0;
-	pos[1] = 0;
-
-	gtk_window_get_size(GTK_WINDOW(wnd), &window_dim[0], &window_dim[1]);
-
-	for(gint i = 0; i < 2; i++)
-	{
-		GType type = G_VALUE_TYPE(&geom_pos[i]);
-
-		if(type == G_TYPE_INT64)
-		{
-			gint64 value =	g_value_get_int64(&geom_pos[i]);
-
-			pos[i] =	flip[i]?
-					screen_dim[i]-window_dim[i]-value:
-					value;
-		}
-		else if(type == G_TYPE_DOUBLE)
-		{
-			gdouble value =	g_value_get_double(&geom_pos[i]);
-
-			pos[i] = (gint64)(value*(screen_dim[i]-window_dim[i]));
-		}
-		else
-		{
-			g_assert_not_reached();
-		}
-	}
-
-	/* Adjust the y-position to account for the height of
-	 * the control box.
-	 */
-	if(flip[1] && gtk_widget_get_visible(GTK_WIDGET(box)))
-	{
-		pos[1] -=	gtk_widget_get_allocated_height
-				(GTK_WIDGET(box));
-	}
-
-	gtk_window_move(GTK_WINDOW(wnd), (gint)pos[0], (gint)pos[1]);
-}
-
 static void gmpv_main_window_class_init(GmpvMainWindowClass *klass)
 {
 	GObjectClass *obj_class = G_OBJECT_CLASS(klass);
@@ -284,10 +237,33 @@ static void gmpv_main_window_class_init(GmpvMainWindowClass *klass)
 			FALSE,
 			G_PARAM_READWRITE );
 	g_object_class_install_property(obj_class, PROP_ALWAYS_FLOATING, pspec);
+
+	g_signal_new(	"button-clicked",
+			G_TYPE_FROM_CLASS(klass),
+			G_SIGNAL_RUN_FIRST,
+			0,
+			NULL,
+			NULL,
+			g_cclosure_marshal_VOID__STRING,
+			G_TYPE_NONE,
+			1,
+			G_TYPE_STRING );
+	g_signal_new(	"seek",
+			G_TYPE_FROM_CLASS(klass),
+			G_SIGNAL_RUN_FIRST,
+			0,
+			NULL,
+			NULL,
+			g_cclosure_marshal_VOID__DOUBLE,
+			G_TYPE_NONE,
+			1,
+			G_TYPE_DOUBLE );
 }
 
 static void gmpv_main_window_init(GmpvMainWindow *wnd)
 {
+	GmpvControlBox *vid_area_control_box = NULL;
+
 	wnd->csd = FALSE;
 	wnd->always_floating = FALSE;
 	wnd->use_floating_controls = FALSE;
@@ -296,6 +272,7 @@ static void gmpv_main_window_init(GmpvMainWindow *wnd)
 	wnd->pre_fs_playlist_visible = FALSE;
 	wnd->playlist_width = PLAYLIST_DEFAULT_WIDTH;
 	wnd->timeout_tag = 0;
+	wnd->track_list = NULL;
 	wnd->header_bar = gmpv_header_bar_new();
 	wnd->main_box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	wnd->vid_area_paned = gtk_paned_new(GTK_ORIENTATION_HORIZONTAL);
@@ -306,9 +283,44 @@ static void gmpv_main_window_init(GmpvMainWindow *wnd)
 	wnd->width_offset = 0;
 	wnd->height_offset = 0;
 
+	vid_area_control_box =	gmpv_video_area_get_control_box
+				(GMPV_VIDEO_AREA(wnd->vid_area));
+
 	g_object_bind_property(	wnd, "title",
 				wnd->vid_area, "title",
 				G_BINDING_DEFAULT );
+	g_object_bind_property(	wnd->control_box, "duration",
+				vid_area_control_box, "duration",
+				G_BINDING_DEFAULT );
+	g_object_bind_property(	wnd->control_box, "pause",
+				vid_area_control_box, "pause",
+				G_BINDING_DEFAULT );
+	g_object_bind_property(	wnd->control_box, "chapters-enabled",
+				vid_area_control_box, "chapters-enabled",
+				G_BINDING_DEFAULT );
+	g_object_bind_property(	wnd->control_box, "time-position",
+				vid_area_control_box, "time-position",
+				G_BINDING_DEFAULT );
+	g_object_bind_property(	wnd->control_box, "volume",
+				vid_area_control_box, "volume",
+				G_BINDING_BIDIRECTIONAL );
+
+	g_signal_connect(	wnd->control_box,
+				"seek",
+				G_CALLBACK(seek_handler),
+				wnd );
+	g_signal_connect(	wnd->control_box,
+				"button-clicked",
+				G_CALLBACK(button_clicked_handler),
+				wnd );
+	g_signal_connect(	vid_area_control_box,
+				"seek",
+				G_CALLBACK(seek_handler),
+				wnd );
+	g_signal_connect(	vid_area_control_box,
+				"button-clicked",
+				G_CALLBACK(button_clicked_handler),
+				wnd );
 
 	gtk_widget_add_events(	wnd->vid_area,
 				GDK_ENTER_NOTIFY_MASK
@@ -330,7 +342,7 @@ static void gmpv_main_window_init(GmpvMainWindow *wnd)
 	gtk_container_add(GTK_CONTAINER(wnd), wnd->main_box);
 }
 
-GtkWidget *gmpv_main_window_new(	GmpvApplication *app,
+GtkWidget *gmpv_main_window_new(	GtkApplication *app,
 					gboolean always_floating )
 {
 	return GTK_WIDGET(g_object_new(	gmpv_main_window_get_type(),
@@ -361,25 +373,10 @@ void gmpv_main_window_set_use_floating_controls(	GmpvMainWindow *wnd,
 {
 	if(floating != wnd->use_floating_controls)
 	{
-		GmpvVideoArea *vid_area = GMPV_VIDEO_AREA(wnd->vid_area);
-		GtkContainer *main_box = GTK_CONTAINER(wnd->main_box);
-
-		if(floating)
-		{
-			g_object_ref(wnd->control_box);
-			gtk_container_remove(main_box, wnd->control_box);
-			gmpv_video_area_set_control_box
-				(vid_area, wnd->control_box);
-			g_object_unref(wnd->control_box);
-
-		}
-		else
-		{
-			g_object_ref(wnd->control_box);
-			gmpv_video_area_set_control_box(vid_area, NULL);
-			gtk_container_add(main_box, wnd->control_box);
-			g_object_unref(wnd->control_box);
-		}
+		gtk_widget_set_visible
+			(wnd->control_box, !floating);
+		gmpv_video_area_set_control_box_visible
+			(GMPV_VIDEO_AREA(wnd->vid_area), floating);
 
 		wnd->use_floating_controls = floating;
 	}
@@ -415,8 +412,24 @@ void gmpv_main_window_set_fullscreen(GmpvMainWindow *wnd, gboolean fullscreen)
 
 		if(!gmpv_main_window_get_csd_enabled(wnd))
 		{
-			gtk_application_window_set_show_menubar
-				(GTK_APPLICATION_WINDOW(wnd), !fullscreen);
+			GtkApplication *app;
+			GMenu *menu;
+
+			app = gtk_window_get_application(GTK_WINDOW(wnd));
+			menu = G_MENU(gtk_application_get_menubar(app));
+
+			/* gtk_application_window_set_show_menubar() cannot be
+			 * used here since it will cause assertion failure when
+			 * opengl-cb is used.
+			 */
+			if(fullscreen)
+			{
+				g_menu_remove_all(menu);
+			}
+			else
+			{
+				gmpv_menu_build_full(menu, wnd->track_list);
+			}
 		}
 
 		gmpv_video_area_set_fullscreen_state(vid_area, fullscreen);
@@ -518,27 +531,11 @@ void gmpv_main_window_load_state(GmpvMainWindow *wnd)
 	}
 }
 
-void gmpv_main_window_set_geometry(	GmpvMainWindow *wnd,
-					const GmpvGeometry *geom )
-{
-	if(geom)
-	{
-		if(!(geom->flags&GMPV_GEOMETRY_IGNORE_DIM))
-		{
-			gmpv_main_window_resize_video_area
-				(wnd, (gint)geom->width, (gint)geom->height);
-		}
-
-		if(!(geom->flags&GMPV_GEOMETRY_IGNORE_POS))
-		{
-			set_window_pos(wnd, geom);
-		}
-	}
-}
-
 void gmpv_main_window_update_track_list(	GmpvMainWindow *wnd,
 						const GPtrArray *track_list )
 {
+	wnd->track_list = track_list;
+
 	if(gmpv_main_window_get_csd_enabled(wnd))
 	{
 		gmpv_header_bar_update_track_list
@@ -686,3 +683,15 @@ gboolean gmpv_main_window_get_playlist_visible(GmpvMainWindow *wnd)
 {
 	return gtk_widget_get_visible(GTK_WIDGET(wnd->playlist));
 }
+
+void gmpv_main_window_set_controls_visible(	GmpvMainWindow *wnd,
+						gboolean visible )
+{
+	gtk_widget_set_visible(GTK_WIDGET(wnd->control_box), visible);
+}
+
+gboolean gmpv_main_window_get_controls_visible(GmpvMainWindow *wnd)
+{
+	return gtk_widget_get_visible(GTK_WIDGET(wnd->control_box));
+}
+
