@@ -27,7 +27,6 @@
 #include "gmpv_playlist_widget.h"
 #include "gmpv_control_box.h"
 #include "gmpv_video_area.h"
-#include "gmpv_geometry.h"
 
 G_BEGIN_DECLS
 
@@ -35,9 +34,7 @@ G_BEGIN_DECLS
 
 G_DECLARE_FINAL_TYPE(GmpvMainWindow, gmpv_main_window, GMPV, MAIN_WINDOW, GtkApplicationWindow)
 
-typedef struct _GmpvApplication GmpvApplication;
-
-GtkWidget *gmpv_main_window_new(	GmpvApplication *app,
+GtkWidget *gmpv_main_window_new(	GtkApplication *app,
 					gboolean always_floating );
 GmpvPlaylistWidget *gmpv_main_window_get_playlist(GmpvMainWindow *wnd);
 GmpvControlBox *gmpv_main_window_get_control_box(GmpvMainWindow *wnd);
@@ -51,8 +48,6 @@ void gmpv_main_window_toggle_fullscreen(GmpvMainWindow *wnd);
 void gmpv_main_window_reset(GmpvMainWindow *wnd);
 void gmpv_main_window_save_state(GmpvMainWindow *wnd);
 void gmpv_main_window_load_state(GmpvMainWindow *wnd);
-void gmpv_main_window_set_geometry(	GmpvMainWindow *wnd,
-					const GmpvGeometry *geom );
 void gmpv_main_window_update_track_list(	GmpvMainWindow *wnd,
 						const GPtrArray *track_list );
 void gmpv_main_window_resize_video_area(	GmpvMainWindow *wnd,
@@ -63,6 +58,9 @@ gboolean gmpv_main_window_get_csd_enabled(GmpvMainWindow *wnd);
 void gmpv_main_window_set_playlist_visible(	GmpvMainWindow *wnd,
 						gboolean visible );
 gboolean gmpv_main_window_get_playlist_visible(GmpvMainWindow *wnd);
+void gmpv_main_window_set_controls_visible(	GmpvMainWindow *wnd,
+						gboolean visible );
+gboolean gmpv_main_window_get_controls_visible(GmpvMainWindow *wnd);
 
 G_END_DECLS
 
