@@ -20,14 +20,19 @@
 #include <epoxy/gl.h>
 
 #include "gmpv_model.h"
-#include "gmpv_mpv.h"
+#include "gmpv_marshal.h"
+#include "gmpv_player.h"
 #include "gmpv_mpv_wrapper.h"
+#include "gmpv_metadata_cache.h"
 
 enum
 {
 	PROP_INVALID,
 	PROP_MPV,
 	PROP_READY,
+	PROP_PLAYLIST,
+	PROP_METADATA,
+	PROP_TRACK_LIST,
 	PROP_AID,
 	PROP_VID,
 	PROP_SID,
@@ -36,15 +41,12 @@ enum
 	PROP_IDLE_ACTIVE,
 	PROP_FULLSCREEN,
 	PROP_PAUSE,
-	PROP_LOOP,
+	PROP_LOOP_PLAYLIST,
 	PROP_DURATION,
 	PROP_MEDIA_TITLE,
-	PROP_METADATA,
-	PROP_PLAYLIST,
 	PROP_PLAYLIST_COUNT,
 	PROP_PLAYLIST_POS,
 	PROP_SPEED,
-	PROP_TRACK_LIST,
 	PROP_VOLUME,
 	N_PROPERTIES
 };
@@ -52,8 +54,12 @@ enum
 struct _GmpvModel
 {
 	GObject parent;
-	GmpvMpv *mpv;
+	GmpvPlayer *player;
 	gboolean ready;
+	GmpvMetadataCache *cache;
+	GPtrArray *playlist;
+	GPtrArray *metadata;
+	GPtrArray *track_list;
 	gboolean update_mpv_properties;
 	gchar *aid;
 	gchar *vid;
@@ -63,15 +69,12 @@ struct _GmpvModel
 	gboolean idle_active;
 	gboolean fullscreen;
 	gboolean pause;
-	gchar *loop;
+	gchar *loop_playlist;
 	gdouble duration;
 	gchar *media_title;
-	GPtrArray *metadata;
-	GPtrArray *playlist;
 	gint64 playlist_count;
 	gint64 playlist_pos;
 	gdouble speed;
-	GPtrArray *track_list;
 	gdouble volume;
 };
 
@@ -89,6 +92,8 @@ static void get_property(	GObject *object,
 				guint property_id,
 				GValue *value,
 				GParamSpec *pspec );
+static void dispose(GObject *object);
+static void finalize(GObject *object);
 static void set_mpv_property(	GObject *object,
 				guint property_id,
 				const GValue *value,
@@ -101,9 +106,19 @@ static GParamSpec *g_param_spec_by_type(	const gchar *name,
 						GParamFlags flags );
 static gboolean emit_frame_ready(gpointer data);
 static void opengl_cb_update_callback(gpointer opengl_cb_ctx);
-static void autofit_handler(GmpvMpv *mpv, gdouble ratio, gpointer data);
-static void mpv_init_handler(GmpvMpv *mpv, gpointer data);
-static void mpv_playback_restart_handler(GmpvMpv *mpv, gpointer data);
+static void metadata_update_handler(	GmpvPlayer *player,
+					gint64 pos,
+					gpointer data );
+static void window_resize_handler(	GmpvMpv *mpv,
+					gint64 width,
+					gint64 height,
+					gpointer data );
+static void window_move_handler(	GmpvMpv *mpv,
+					gboolean flip_x,
+					gboolean flip_y,
+					GValue *x,
+					GValue *y,
+					gpointer data );
 static void mpv_prop_change_handler(	GmpvMpv *mpv,
 					const gchar *name,
 					gpointer value,
@@ -118,39 +133,59 @@ static void constructed(GObject *object)
 {
 	GmpvModel *model = GMPV_MODEL(object);
 
-	g_assert(model->mpv);
+	g_assert(model->player);
 
-	model->playlist = gmpv_mpv_get_playlist(model->mpv);
-	model->metadata = gmpv_mpv_get_metadata(model->mpv);
+	g_object_bind_property(	model->player,
+				"ready",
+				model,
+				"ready",
+				G_BINDING_DEFAULT );
+	g_object_bind_property(	model->player,
+				"playlist",
+				model,
+				"playlist",
+				G_BINDING_DEFAULT|G_BINDING_SYNC_CREATE );
+	g_object_bind_property(	model->player,
+				"metadata",
+				model,
+				"metadata",
+				G_BINDING_DEFAULT|G_BINDING_SYNC_CREATE );
+	g_object_bind_property(	model->player,
+				"track-list",
+				model,
+				"track-list",
+				G_BINDING_DEFAULT|G_BINDING_SYNC_CREATE );
 
-	g_signal_connect(	model->mpv,
-				"autofit",
-				G_CALLBACK(autofit_handler),
+	g_signal_connect(	model->player,
+				"metadata-update",
+				G_CALLBACK(metadata_update_handler),
 				model );
-	g_signal_connect(	model->mpv,
-				"mpv-init",
-				G_CALLBACK(mpv_init_handler),
+	g_signal_connect(	model->player,
+				"window-resize",
+				G_CALLBACK(window_resize_handler),
 				model );
-	g_signal_connect(	model->mpv,
-				"mpv-playback-restart",
-				G_CALLBACK(mpv_playback_restart_handler),
+	g_signal_connect(	model->player,
+				"window-move",
+				G_CALLBACK(window_move_handler),
 				model );
-	g_signal_connect(	model->mpv,
-				"mpv-prop-change",
+	g_signal_connect(	model->player,
+				"mpv-property-changed",
 				G_CALLBACK(mpv_prop_change_handler),
 				model );
-	g_signal_connect(	model->mpv,
+	g_signal_connect(	model->player,
 				"error",
 				G_CALLBACK(error_handler),
 				model );
-	g_signal_connect(	model->mpv,
+	g_signal_connect(	model->player,
 				"message",
 				G_CALLBACK(message_handler),
 				model );
-	g_signal_connect(	model->mpv,
+	g_signal_connect(	model->player,
 				"shutdown",
 				G_CALLBACK(shutdown_handler),
 				model );
+
+	G_OBJECT_CLASS(gmpv_model_parent_class)->constructed(object);
 }
 
 static void set_property(	GObject *object,
@@ -163,13 +198,25 @@ static void set_property(	GObject *object,
 	switch(property_id)
 	{
 		case PROP_MPV:
-		self->mpv = g_value_get_pointer(value);
+		self->player = g_value_get_pointer(value);
 		break;
 
 		case PROP_READY:
 		self->ready = g_value_get_boolean(value);
 		break;
 
+		case PROP_PLAYLIST:
+		self->playlist = g_value_get_pointer(value);
+		break;
+
+		case PROP_METADATA:
+		self->metadata = g_value_get_pointer(value);
+		break;
+
+		case PROP_TRACK_LIST:
+		self->track_list = g_value_get_pointer(value);
+		break;
+
 		case PROP_AID:
 		g_free(self->aid);
 		self->aid = g_value_dup_string(value);
@@ -205,8 +252,8 @@ static void set_property(	GObject *object,
 		self->pause = g_value_get_boolean(value);
 		break;
 
-		case PROP_LOOP:
-		self->loop = g_value_dup_string(value);
+		case PROP_LOOP_PLAYLIST:
+		self->loop_playlist = g_value_dup_string(value);
 		break;
 
 		case PROP_DURATION:
@@ -218,14 +265,6 @@ static void set_property(	GObject *object,
 		self->media_title = g_value_dup_string(value);
 		break;
 
-		case PROP_METADATA:
-		self->metadata = gmpv_mpv_get_metadata(self->mpv);
-		break;
-
-		case PROP_PLAYLIST:
-		self->playlist = gmpv_mpv_get_playlist(self->mpv);
-		break;
-
 		case PROP_PLAYLIST_COUNT:
 		self->playlist_count = g_value_get_int64(value);
 		break;
@@ -238,10 +277,6 @@ static void set_property(	GObject *object,
 		self->speed = g_value_get_double(value);
 		break;
 
-		case PROP_TRACK_LIST:
-		self->track_list = gmpv_mpv_get_track_list(self->mpv);
-		break;
-
 		case PROP_VOLUME:
 		self->volume = g_value_get_double(value);
 		break;
@@ -268,13 +303,25 @@ static void get_property(	GObject *object,
 	switch(property_id)
 	{
 		case PROP_MPV:
-		g_value_set_pointer(value, self->mpv);
+		g_value_set_pointer(value, self->player);
 		break;
 
 		case PROP_READY:
 		g_value_set_boolean(value, self->ready);
 		break;
 
+		case PROP_PLAYLIST:
+		g_value_set_pointer(value, self->playlist);
+		break;
+
+		case PROP_METADATA:
+		g_value_set_pointer(value, self->metadata);
+		break;
+
+		case PROP_TRACK_LIST:
+		g_value_set_pointer(value, self->track_list);
+		break;
+
 		case PROP_AID:
 		g_value_set_string(value, self->aid);
 		break;
@@ -307,8 +354,8 @@ static void get_property(	GObject *object,
 		g_value_set_boolean(value, self->pause);
 		break;
 
-		case PROP_LOOP:
-		g_value_set_string(value, self->loop);
+		case PROP_LOOP_PLAYLIST:
+		g_value_set_string(value, self->loop_playlist);
 		break;
 
 		case PROP_DURATION:
@@ -319,14 +366,6 @@ static void get_property(	GObject *object,
 		g_value_set_string(value, self->media_title);
 		break;
 
-		case PROP_METADATA:
-		g_value_set_pointer(value, self->metadata);
-		break;
-
-		case PROP_PLAYLIST:
-		g_value_set_pointer(value, self->playlist);
-		break;
-
 		case PROP_PLAYLIST_COUNT:
 		g_value_set_int64(value, self->playlist_count);
 		break;
@@ -339,10 +378,6 @@ static void get_property(	GObject *object,
 		g_value_set_double(value, self->speed);
 		break;
 
-		case PROP_TRACK_LIST:
-		g_value_set_pointer(value, self->track_list);
-		break;
-
 		case PROP_VOLUME:
 		g_value_set_double(value, self->volume);
 		break;
@@ -353,80 +388,109 @@ static void get_property(	GObject *object,
 	}
 }
 
+static void dispose(GObject *object)
+{
+	GmpvModel *model = GMPV_MODEL(object);
+	GmpvMpv *mpv = GMPV_MPV(model->player);
+
+	if(mpv)
+	{
+		gmpv_mpv_set_opengl_cb_callback(mpv, NULL, NULL);
+		g_clear_object(&model->player);
+		while(g_source_remove_by_user_data(model));
+	}
+
+	G_OBJECT_CLASS(gmpv_model_parent_class)->dispose(object);
+}
+
+static void finalize(GObject *object)
+{
+	GmpvModel *model = GMPV_MODEL(object);
+
+	g_free(model->aid);
+	g_free(model->vid);
+	g_free(model->sid);
+	g_free(model->loop_playlist);
+	g_free(model->media_title);
+
+	G_OBJECT_CLASS(gmpv_model_parent_class)->finalize(object);
+}
+
 static void set_mpv_property(	GObject *object,
 				guint property_id,
 				const GValue *value,
 				GParamSpec *pspec )
 {
 	GmpvModel *self = GMPV_MODEL(object);
+	GmpvMpv *mpv = GMPV_MPV(self->player);
 
 	switch(property_id)
 	{
 		case PROP_AID:
-		gmpv_mpv_set_property(	self->mpv,
+		gmpv_mpv_set_property(	mpv,
 					"aid",
 					MPV_FORMAT_STRING,
 					&self->aid );
 		break;
 
 		case PROP_VID:
-		gmpv_mpv_set_property(	self->mpv,
+		gmpv_mpv_set_property(	mpv,
 					"vid",
 					MPV_FORMAT_STRING,
 					&self->vid );
 		break;
 
 		case PROP_SID:
-		gmpv_mpv_set_property(	self->mpv,
+		gmpv_mpv_set_property(	mpv,
 					"sid",
 					MPV_FORMAT_STRING,
 					&self->sid );
 		break;
 
 		case PROP_FULLSCREEN:
-		gmpv_mpv_set_property(	self->mpv,
+		gmpv_mpv_set_property(	mpv,
 					"fullscreen",
 					MPV_FORMAT_FLAG,
 					&self->fullscreen );
 		break;
 
 		case PROP_PAUSE:
-		gmpv_mpv_set_property(	self->mpv,
+		gmpv_mpv_set_property(	mpv,
 					"pause",
 					MPV_FORMAT_FLAG,
 					&self->pause );
 		break;
 
-		case PROP_LOOP:
-		gmpv_mpv_set_property(	self->mpv,
-					"loop",
+		case PROP_LOOP_PLAYLIST:
+		gmpv_mpv_set_property(	mpv,
+					"loop-playlist",
 					MPV_FORMAT_STRING,
-					&self->loop );
+					&self->loop_playlist );
 		break;
 
 		case PROP_MEDIA_TITLE:
-		gmpv_mpv_set_property(	self->mpv,
+		gmpv_mpv_set_property(	mpv,
 					"media-title",
 					MPV_FORMAT_INT64,
 					&self->media_title );
 		break;
 
 		case PROP_PLAYLIST_POS:
-		gmpv_mpv_set_property(	self->mpv,
+		gmpv_mpv_set_property(	mpv,
 					"playlist-pos",
 					MPV_FORMAT_INT64,
 					&self->playlist_pos );
 		break;
 
 		case PROP_SPEED:
-		gmpv_mpv_set_property(	self->mpv,
+		gmpv_mpv_set_property(	mpv,
 					"speed",
 					MPV_FORMAT_DOUBLE,
 					&self->speed );
 		break;
 
 		case PROP_VOLUME:
-		gmpv_mpv_set_property(	self->mpv,
+		gmpv_mpv_set_property(	mpv,
 					"volume",
 					MPV_FORMAT_DOUBLE,
 					&self->volume );
@@ -532,20 +596,29 @@ static void opengl_cb_update_callback(gpointer data)
 				NULL );
 }
 
-static void autofit_handler(GmpvMpv *mpv, gdouble ratio, gpointer data)
+static void metadata_update_handler(	GmpvPlayer *player,
+					gint64 pos,
+					gpointer data )
 {
-	g_signal_emit_by_name(data, "autofit", ratio);
+	g_signal_emit_by_name(data, "metadata-update", pos);
 }
 
-static void mpv_init_handler(GmpvMpv *mpv, gpointer data)
+static void window_resize_handler(	GmpvMpv *mpv,
+					gint64 width,
+					gint64 height,
+					gpointer data )
 {
-	GMPV_MODEL(data)->ready = TRUE;
-	g_object_notify(data, "ready");
+	g_signal_emit_by_name(data, "window-resize", width, height);
 }
 
-static void mpv_playback_restart_handler(GmpvMpv *mpv, gpointer data)
+static void window_move_handler(	GmpvMpv *mpv,
+					gboolean flip_x,
+					gboolean flip_y,
+					GValue *x,
+					GValue *y,
+					gpointer data )
 {
-	g_signal_emit_by_name(data, "playback-restart");
+	g_signal_emit_by_name(data, "window-move", flip_x, flip_y, x, y);
 }
 
 static void mpv_prop_change_handler(	GmpvMpv *mpv,
@@ -553,23 +626,28 @@ static void mpv_prop_change_handler(	GmpvMpv *mpv,
 					gpointer value,
 					gpointer data )
 {
-	GObjectClass *klass;
-	GParamSpec *pspec;
-	GValue gvalue = G_VALUE_INIT;
-
-	klass = G_TYPE_INSTANCE_GET_CLASS(data, GMPV_TYPE_MODEL, GObjectClass);
-	pspec = g_object_class_find_property(klass, name);
-
-	if(pspec && value)
+	if(	g_strcmp0(name, "playlist") != 0 &&
+		g_strcmp0(name, "metadata") != 0 &&
+		g_strcmp0(name, "track-list") != 0 )
 	{
-		GMPV_MODEL(data)->update_mpv_properties = FALSE;
+		GObjectClass *klass;
+		GParamSpec *pspec;
+		GValue gvalue = G_VALUE_INIT;
 
-		g_value_set_by_type(&gvalue, pspec->value_type, value);
-		g_object_set_property(data, name, &gvalue);
+		klass =	G_TYPE_INSTANCE_GET_CLASS
+			(data, GMPV_TYPE_MODEL, GObjectClass);
+		pspec = g_object_class_find_property(klass, name);
 
-		GMPV_MODEL(data)->update_mpv_properties = TRUE;
+		if(pspec && value)
+		{
+			GMPV_MODEL(data)->update_mpv_properties = FALSE;
+
+			g_value_set_by_type(&gvalue, pspec->value_type, value);
+			g_object_set_property(data, name, &gvalue);
+
+			GMPV_MODEL(data)->update_mpv_properties = TRUE;
+		}
 	}
-
 }
 
 static void error_handler(GmpvMpv *mpv, const gchar *message, gpointer data)
@@ -607,15 +685,12 @@ static void gmpv_model_class_init(GmpvModelClass *klass)
 			{"idle-active", PROP_IDLE_ACTIVE, G_TYPE_BOOLEAN},
 			{"fullscreen", PROP_FULLSCREEN, G_TYPE_BOOLEAN},
 			{"pause", PROP_PAUSE, G_TYPE_BOOLEAN},
-			{"loop", PROP_LOOP, G_TYPE_STRING},
+			{"loop-playlist", PROP_LOOP_PLAYLIST, G_TYPE_STRING},
 			{"duration", PROP_DURATION, G_TYPE_DOUBLE},
 			{"media-title", PROP_MEDIA_TITLE, G_TYPE_STRING},
-			{"metadata", PROP_METADATA, G_TYPE_POINTER},
-			{"playlist", PROP_PLAYLIST, G_TYPE_POINTER},
 			{"playlist-count", PROP_PLAYLIST_COUNT, G_TYPE_INT64},
 			{"playlist-pos", PROP_PLAYLIST_POS, G_TYPE_INT64},
 			{"speed", PROP_SPEED, G_TYPE_DOUBLE},
-			{"track-list", PROP_TRACK_LIST, G_TYPE_POINTER},
 			{"volume", PROP_VOLUME, G_TYPE_DOUBLE},
 			{NULL, PROP_INVALID, 0} };
 
@@ -625,6 +700,8 @@ static void gmpv_model_class_init(GmpvModelClass *klass)
 	obj_class->constructed = constructed;
 	obj_class->set_property = set_property;
 	obj_class->get_property = get_property;
+	obj_class->dispose = dispose;
+	obj_class->finalize = finalize;
 
 	pspec = g_param_spec_pointer
 		(	"mpv",
@@ -638,9 +715,30 @@ static void gmpv_model_class_init(GmpvModelClass *klass)
 			"Ready",
 			"Whether mpv is ready to receive commands",
 			FALSE,
-			G_PARAM_READABLE );
+			G_PARAM_READWRITE );
 	g_object_class_install_property(obj_class, PROP_READY, pspec);
 
+	pspec = g_param_spec_pointer
+		(	"playlist",
+			"playlist",
+			"The playlist",
+			G_PARAM_READWRITE );
+	g_object_class_install_property(obj_class, PROP_PLAYLIST, pspec);
+
+	pspec = g_param_spec_pointer
+		(	"metadata",
+			"metadata",
+			"Metadata tags of the current file",
+			G_PARAM_READWRITE );
+	g_object_class_install_property(obj_class, PROP_METADATA, pspec);
+
+	pspec = g_param_spec_pointer
+		(	"track-list",
+			"track-list",
+			"Audio, video, and subtitle tracks of the current file",
+			G_PARAM_READWRITE );
+	g_object_class_install_property(obj_class, PROP_TRACK_LIST, pspec);
+
 	for(int i = 0; mpv_props[i].name; i++)
 	{
 		pspec = g_param_spec_by_type(	mpv_props[i].name,
@@ -670,16 +768,40 @@ static void gmpv_model_class_init(GmpvModelClass *klass)
 			g_cclosure_marshal_VOID__VOID,
 			G_TYPE_NONE,
 			0 );
-	g_signal_new(	"autofit",
+	g_signal_new(	"metadata-update",
 			G_TYPE_FROM_CLASS(klass),
 			G_SIGNAL_RUN_FIRST,
 			0,
 			NULL,
 			NULL,
-			g_cclosure_marshal_VOID__DOUBLE,
+			g_cclosure_gen_marshal_VOID__INT64,
 			G_TYPE_NONE,
 			1,
-			G_TYPE_DOUBLE );
+			G_TYPE_INT64 );
+	g_signal_new(	"window-resize",
+			G_TYPE_FROM_CLASS(klass),
+			G_SIGNAL_RUN_FIRST,
+			0,
+			NULL,
+			NULL,
+			g_cclosure_gen_marshal_VOID__INT64_INT64,
+			G_TYPE_NONE,
+			2,
+			G_TYPE_INT64,
+			G_TYPE_INT64 );
+	g_signal_new(	"window-move",
+			G_TYPE_FROM_CLASS(klass),
+			G_SIGNAL_RUN_FIRST,
+			0,
+			NULL,
+			NULL,
+			g_cclosure_gen_marshal_VOID__BOOLEAN_BOOLEAN_POINTER_POINTER,
+			G_TYPE_NONE,
+			4,
+			G_TYPE_BOOLEAN,
+			G_TYPE_BOOLEAN,
+			G_TYPE_POINTER,
+			G_TYPE_POINTER );
 	g_signal_new(	"message",
 			G_TYPE_FROM_CLASS(klass),
 			G_SIGNAL_RUN_FIRST,
@@ -713,8 +835,12 @@ static void gmpv_model_class_init(GmpvModelClass *klass)
 
 static void gmpv_model_init(GmpvModel *model)
 {
-	model->mpv = NULL;
+	model->player = NULL;
 	model->ready = FALSE;
+	model->cache = NULL;
+	model->playlist = NULL;
+	model->metadata = NULL;
+	model->track_list = NULL;
 	model->update_mpv_properties = TRUE;
 	model->aid = NULL;
 	model->vid = NULL;
@@ -724,47 +850,47 @@ static void gmpv_model_init(GmpvModel *model)
 	model->idle_active = FALSE;
 	model->fullscreen = FALSE;
 	model->pause = TRUE;
-	model->loop = NULL;
+	model->loop_playlist = NULL;
 	model->duration = 0.0;
 	model->media_title = NULL;
-	model->metadata = NULL;
-	model->playlist = NULL;
 	model->playlist_count = 0;
 	model->playlist_pos = 0;
 	model->speed = 1.0;
-	model->track_list = NULL;
 	model->volume = 1.0;
 }
 
-GmpvModel *gmpv_model_new(GmpvMpv *mpv)
+GmpvModel *gmpv_model_new(gint64 wid)
 {
 	return GMPV_MODEL(g_object_new(	gmpv_model_get_type(),
-					"mpv", mpv,
+					"mpv", gmpv_player_new(wid),
 					NULL ));
 }
 
 void gmpv_model_initialize(GmpvModel *model)
 {
-	gmpv_mpv_initialize(model->mpv);
+	GmpvMpv *mpv = GMPV_MPV(model->player);
 
-	gmpv_mpv_set_opengl_cb_callback
-		(model->mpv, opengl_cb_update_callback, model);
+	gmpv_mpv_initialize(mpv);
+	gmpv_mpv_set_opengl_cb_callback(mpv, opengl_cb_update_callback, model);
 }
 
 void gmpv_model_reset(GmpvModel *model)
 {
-	GMPV_MODEL(model)->ready = FALSE;
+	model->ready = FALSE;
 	g_object_notify(G_OBJECT(model), "ready");
 
-	gmpv_mpv_reset(model->mpv);
+	gmpv_mpv_reset(GMPV_MPV(model->player));
 }
 
 void gmpv_model_quit(GmpvModel *model)
 {
-	GMPV_MODEL(model)->ready = FALSE;
-	g_object_notify(G_OBJECT(model), "ready");
+	if(model->ready)
+	{
+		model->ready = FALSE;
+		g_object_notify(G_OBJECT(model), "ready");
 
-	gmpv_mpv_quit(model->mpv);
+		gmpv_mpv_quit(GMPV_MPV(model->player));
+	}
 }
 
 void gmpv_model_mouse(GmpvModel *model, gint x, gint y)
@@ -774,7 +900,10 @@ void gmpv_model_mouse(GmpvModel *model, gint x, gint y)
 	const gchar *cmd[] = {"mouse", x_str, y_str, NULL};
 
 	g_debug("Set mouse location to (%s, %s)", x_str, y_str);
-	gmpv_mpv_command(model->mpv, cmd);
+	gmpv_mpv_command(GMPV_MPV(model->player), cmd);
+
+	g_free(x_str);
+	g_free(y_str);
 }
 
 void gmpv_model_key_down(GmpvModel *model, const gchar* keystr)
@@ -782,7 +911,7 @@ void gmpv_model_key_down(GmpvModel *model, const gchar* keystr)
 	const gchar *cmd[] = {"keydown", keystr, NULL};
 
 	g_debug("Sent '%s' key down to mpv", keystr);
-	gmpv_mpv_command(model->mpv, cmd);
+	gmpv_mpv_command(GMPV_MPV(model->player), cmd);
 }
 
 void gmpv_model_key_up(GmpvModel *model, const gchar* keystr)
@@ -790,7 +919,7 @@ void gmpv_model_key_up(GmpvModel *model, const gchar* keystr)
 	const gchar *cmd[] = {"keyup", keystr, NULL};
 
 	g_debug("Sent '%s' key up to mpv", keystr);
-	gmpv_mpv_command(model->mpv, cmd);
+	gmpv_mpv_command(GMPV_MPV(model->player), cmd);
 }
 
 void gmpv_model_key_press(GmpvModel *model, const gchar* keystr)
@@ -798,77 +927,84 @@ void gmpv_model_key_press(GmpvModel *model, const gchar* keystr)
 	const gchar *cmd[] = {"keypress", keystr, NULL};
 
 	g_debug("Sent '%s' key press to mpv", keystr);
-	gmpv_mpv_command(model->mpv, cmd);
+	gmpv_mpv_command(GMPV_MPV(model->player), cmd);
 }
 
 void gmpv_model_reset_keys(GmpvModel *model)
 {
 	g_debug("Sent global key up to mpv");
-	gmpv_mpv_command_string(model->mpv, "keyup");
+	gmpv_mpv_command_string(GMPV_MPV(model->player), "keyup");
 }
 
 void gmpv_model_play(GmpvModel *model)
 {
-	gmpv_mpv_set_property_flag(model->mpv, "pause", FALSE);
+	gmpv_mpv_set_property_flag(GMPV_MPV(model->player), "pause", FALSE);
 }
 
 void gmpv_model_pause(GmpvModel *model)
 {
-	gmpv_mpv_set_property_flag(model->mpv, "pause", TRUE);
+	gmpv_mpv_set_property_flag(GMPV_MPV(model->player), "pause", TRUE);
 }
 
 void gmpv_model_stop(GmpvModel *model)
 {
 	const gchar *cmd[] = {"stop", NULL};
 
-	gmpv_mpv_command(model->mpv, cmd);
+	gmpv_mpv_command(GMPV_MPV(model->player), cmd);
 }
 
 void gmpv_model_forward(GmpvModel *model)
 {
 	const gchar *cmd[] = {"seek", "10", NULL};
 
-	gmpv_mpv_command(model->mpv, cmd);
+	gmpv_mpv_command(GMPV_MPV(model->player), cmd);
 }
 
 void gmpv_model_rewind(GmpvModel *model)
 {
 	const gchar *cmd[] = {"seek", "-10", NULL};
 
-	gmpv_mpv_command(model->mpv, cmd);
+	gmpv_mpv_command(GMPV_MPV(model->player), cmd);
 }
 
 void gmpv_model_next_chapter(GmpvModel *model)
 {
 	const gchar *cmd[] = {"osd-msg", "cycle", "chapter", NULL};
 
-	gmpv_mpv_command(model->mpv, cmd);
+	gmpv_mpv_command(GMPV_MPV(model->player), cmd);
 }
 
 void gmpv_model_previous_chapter(GmpvModel *model)
 {
 	const gchar *cmd[] = {"osd-msg", "cycle", "chapter", "down", NULL};
 
-	gmpv_mpv_command(model->mpv, cmd);
+	gmpv_mpv_command(GMPV_MPV(model->player), cmd);
 }
 
 void gmpv_model_next_playlist_entry(GmpvModel *model)
 {
 	const gchar *cmd[] = {"osd-msg", "playlist-next", "weak", NULL};
 
-	gmpv_mpv_command(model->mpv, cmd);
+	gmpv_mpv_command(GMPV_MPV(model->player), cmd);
 }
 
 void gmpv_model_previous_playlist_entry(GmpvModel *model)
 {
 	const gchar *cmd[] = {"osd-msg", "playlist-prev", "weak", NULL};
 
-	gmpv_mpv_command(model->mpv, cmd);
+	gmpv_mpv_command(GMPV_MPV(model->player), cmd);
+}
+
+void gmpv_model_shuffle_playlist(GmpvModel *model)
+{
+	const gchar *cmd[] = {"osd-msg", "playlist-shuffle", NULL};
+
+	gmpv_mpv_command(GMPV_MPV(model->player), cmd);
 }
 
 void gmpv_model_seek(GmpvModel *model, gdouble value)
 {
-	gmpv_mpv_set_property(model->mpv, "time-pos", MPV_FORMAT_DOUBLE, &value);
+	gmpv_mpv_set_property(GMPV_MPV(model->player), "time-pos", MPV_FORMAT_DOUBLE, &value);
 }
 
 void gmpv_model_seek_offset(GmpvModel *model, gdouble offset)
@@ -879,17 +1015,19 @@ void gmpv_model_seek_offset(GmpvModel *model, gdouble offset)
 	g_ascii_dtostr(buf, G_ASCII_DTOSTR_BUF_SIZE, offset);
 	cmd[1] = buf;
 
-	gmpv_mpv_command(model->mpv, cmd);
+	gmpv_mpv_command(GMPV_MPV(model->player), cmd);
 }
 
 void gmpv_model_load_audio_track(GmpvModel *model, const gchar *filename)
 {
-	gmpv_mpv_load_track(model->mpv, filename, TRACK_TYPE_AUDIO);
+	gmpv_mpv_load_track
+		(GMPV_MPV(model->player), filename, TRACK_TYPE_AUDIO);
 }
 
 void gmpv_model_load_subtitle_track(GmpvModel *model, const gchar *filename)
 {
-	gmpv_mpv_load_track(model->mpv, filename, TRACK_TYPE_SUBTITLE);
+	gmpv_mpv_load_track
+		(GMPV_MPV(model->player), filename, TRACK_TYPE_SUBTITLE);
 }
 
 gdouble gmpv_model_get_time_position(GmpvModel *model)
@@ -898,7 +1036,7 @@ gdouble gmpv_model_get_time_position(GmpvModel *model)
 
 	if(!model->idle_active)
 	{
-		gmpv_mpv_get_property(	model->mpv,
+		gmpv_mpv_get_property(	GMPV_MPV(model->player),
 					"time-pos",
 					MPV_FORMAT_DOUBLE,
 					&time_pos );
@@ -912,7 +1050,7 @@ void gmpv_model_set_playlist_position(GmpvModel *model, gint64 position)
 {
 	if(position != model->playlist_pos)
 	{
-		gmpv_mpv_set_property(	model->mpv,
+		gmpv_mpv_set_property(	GMPV_MPV(model->player),
 					"playlist-pos",
 					MPV_FORMAT_INT64,
 					&position );
@@ -926,7 +1064,7 @@ void gmpv_model_remove_playlist_entry(GmpvModel *model, gint64 position)
 
 	cmd[1] = index_str;
 
-	gmpv_mpv_command(model->mpv, cmd);
+	gmpv_mpv_command(GMPV_MPV(model->player), cmd);
 
 	g_free(index_str);
 }
@@ -942,7 +1080,7 @@ void gmpv_model_move_playlist_entry(GmpvModel *model, gint64 src, gint64 dst)
 	cmd[1] = src_str;
 	cmd[2] = dst_str;
 
-	gmpv_mpv_command(model->mpv, cmd);
+	gmpv_mpv_command(GMPV_MPV(model->player), cmd);
 
 	g_free(src_str);
 	g_free(dst_str);
@@ -950,7 +1088,7 @@ void gmpv_model_move_playlist_entry(GmpvModel *model, gint64 src, gint64 dst)
 
 void gmpv_model_load_file(GmpvModel *model, const gchar *uri, gboolean append)
 {
-	gmpv_mpv_load(model->mpv, uri, append);
+	gmpv_mpv_load(GMPV_MPV(model->player), uri, append);
 
 	/* Start playing when replacing the playlist, ie. not appending, or
 	 * adding the first file to the playlist.
@@ -963,19 +1101,19 @@ void gmpv_model_load_file(GmpvModel *model, const gchar *uri, gboolean append)
 
 gboolean gmpv_model_get_use_opengl_cb(GmpvModel *model)
 {
-	return gmpv_mpv_get_use_opengl_cb(model->mpv);
+	return gmpv_mpv_get_use_opengl_cb(GMPV_MPV(model->player));
 }
 
 void gmpv_model_initialize_gl(GmpvModel *model)
 {
-	gmpv_mpv_init_gl(model->mpv);
+	gmpv_mpv_init_gl(GMPV_MPV(model->player));
 }
 
 void gmpv_model_render_frame(GmpvModel *model, gint width, gint height)
 {
 	mpv_opengl_cb_context *opengl_ctx;
 
-	opengl_ctx = gmpv_mpv_get_opengl_cb_context(model->mpv);
+	opengl_ctx = gmpv_mpv_get_opengl_cb_context(GMPV_MPV(model->player));
 
 	if(opengl_ctx)
 	{
@@ -990,13 +1128,16 @@ void gmpv_model_get_video_geometry(	GmpvModel *model,
 					gint64 *width,
 					gint64 *height )
 {
-	gmpv_mpv_get_property(model->mpv, "dwidth", MPV_FORMAT_INT64, width);
-	gmpv_mpv_get_property(model->mpv, "dheight", MPV_FORMAT_INT64, height);
+	GmpvMpv *mpv = GMPV_MPV(model->player);
+
+	gmpv_mpv_get_property(mpv, "dwidth", MPV_FORMAT_INT64, width);
+	gmpv_mpv_get_property(mpv, "dheight", MPV_FORMAT_INT64, height);
 }
 
 gchar *gmpv_model_get_current_path(GmpvModel *model)
 {
-	gchar *path = gmpv_mpv_get_property_string(model->mpv, "path");
+	GmpvMpv *mpv = GMPV_MPV(model->player);
+	gchar *path = gmpv_mpv_get_property_string(mpv, "path");
 	gchar *buf = g_strdup(path);
 
 	mpv_free(path);
