@@ -17,43 +17,43 @@
  * along with GNOME MPV.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef MPV_OBJ_H
-#define MPV_OBJ_H
+#ifndef MPV_H
+#define MPV_H
 
 #include <glib.h>
 #include <glib-object.h>
 #include <mpv/client.h>
 #include <mpv/opengl_cb.h>
 
-#include "gmpv_geometry.h"
 #include "gmpv_common.h"
 
 G_BEGIN_DECLS
 
-#define GMPV_TYPE_MPV_OBJ (gmpv_mpv_get_type())
+#define GMPV_TYPE_MPV (gmpv_mpv_get_type())
 
-G_DECLARE_FINAL_TYPE(GmpvMpv, gmpv_mpv, GMPV, MPV_OBJ, GObject)
+G_DECLARE_DERIVABLE_TYPE(GmpvMpv, gmpv_mpv, GMPV, MPV, GObject)
 
-typedef struct _GmpvMpvState GmpvMpvState;
-
-struct _GmpvMpvState
+struct _GmpvMpvClass
 {
-	gboolean ready;
-	gboolean paused;
-	gboolean loaded;
-	gboolean new_file;
-	gboolean init_load;
+	GObjectClass parent_class;
+	void (*mpv_event_notify)(	GmpvMpv *mpv,
+					gint event_id,
+					gpointer event_data );
+	void (*mpv_log_message)(	GmpvMpv *mpv,
+					mpv_log_level log_level,
+					const gchar *prefix,
+					const gchar *text );
+	void (*mpv_property_changed)(	GmpvMpv *mpv,
+					const gchar *name,
+					gpointer value );
+	void (*initialize)(GmpvMpv *mpv);
+	void (*load_file)(GmpvMpv *mpv, const gchar *uri, gboolean append);
+	void (*reset)(GmpvMpv *mpv);
 };
 
 GmpvMpv *gmpv_mpv_new(gint64 wid);
-const GmpvMpvState *gmpv_mpv_get_state(GmpvMpv *mpv);
-GmpvGeometry *gmpv_mpv_get_geometry(GmpvMpv *mpv);
-mpv_handle *gmpv_mpv_get_mpv_handle(GmpvMpv *mpv);
 mpv_opengl_cb_context *gmpv_mpv_get_opengl_cb_context(GmpvMpv *mpv);
 gboolean gmpv_mpv_get_use_opengl_cb(GmpvMpv *mpv);
-GPtrArray *gmpv_mpv_get_metadata(GmpvMpv *mpv);
-GPtrArray *gmpv_mpv_get_playlist(GmpvMpv *mpv);
-GPtrArray *gmpv_mpv_get_track_list(GmpvMpv *mpv);
 void gmpv_mpv_initialize(GmpvMpv *mpv);
 void gmpv_mpv_init_gl(GmpvMpv *mpv);
 void gmpv_mpv_reset(GmpvMpv *mpv);
@@ -61,7 +61,6 @@ void gmpv_mpv_quit(GmpvMpv *mpv);
 void gmpv_mpv_load_track(GmpvMpv *mpv, const gchar *uri, TrackType type);
 void gmpv_mpv_load_file(GmpvMpv *mpv, const gchar *uri, gboolean append);
 void gmpv_mpv_load(GmpvMpv *mpv, const gchar *uri, gboolean append);
-void gmpv_mpv_free(gpointer data);
 
 G_END_DECLS
 
