@@ -22,31 +22,25 @@
 
 #include <gtk/gtk.h>
 
-#include "gmpv_mpv_opt.h"
-
 G_BEGIN_DECLS
 
+typedef struct _GmpvMpvPrivate GmpvMpvPrivate;
+
 enum
 {
 	PROP_0,
 	PROP_WID,
+	PROP_READY,
 	N_PROPERTIES
 };
 
-struct _GmpvMpv
+struct _GmpvMpvPrivate
 {
-	GObject parent;
-	GmpvMpvState state;
 	mpv_handle *mpv_ctx;
 	mpv_opengl_cb_context *opengl_ctx;
-	GPtrArray *playlist;
-	GPtrArray *metadata;
-	GPtrArray *track_list;
-	GtkGLArea *glarea;
+	gboolean ready;
 	gchar *tmp_input_file;
 	GSList *log_level_list;
-	gdouble autofit_ratio;
-	GmpvGeometry *geometry;
 	gboolean init_vo_config;
 	gboolean force_opengl;
 	gboolean use_opengl;
@@ -55,10 +49,8 @@ struct _GmpvMpv
 	void (*opengl_cb_callback)(void *data);
 };
 
-struct _GmpvMpvClass
-{
-	GObjectClass parent_class;
-};
+#define get_private(mpv) \
+	G_TYPE_INSTANCE_GET_PRIVATE(mpv, GMPV_TYPE_MPV, GmpvMpvPrivate)
 
 void mpv_check_error(int status);
 
