@@ -25,6 +25,7 @@
 #include <glib/gi18n.h>
 
 #include "gmpv_playlist_widget.h"
+#include "gmpv_metadata_cache.h"
 #include "gmpv_marshal.h"
 #include "gmpv_common.h"
 #include "gmpv_def.h"
@@ -522,6 +523,7 @@ static gboolean mouse_press_handler(	GtkWidget *widget,
 		GMenu *menu;
 		GMenuItem *add_menu_item;
 		GMenuItem *add_loc_menu_item;
+		GMenuItem *shuffle_menu_item;
 		GMenuItem *loop_menu_item;
 		GtkWidget *ctx_menu;
 
@@ -529,15 +531,17 @@ static gboolean mouse_press_handler(	GtkWidget *widget,
 		add_menu_item
 			=	g_menu_item_new
 				(	_("_Add…"),
-					"app.show-open-dialog(true)" );
+					"win.show-open-dialog(true)" );
 		add_loc_menu_item
 			=	g_menu_item_new
 				(	_("Add _Location…"),
-					"app.show-open-location-dialog(true)" );
-		loop_menu_item = g_menu_item_new(_("Loop"), "app.toggle-loop");
+					"win.show-open-location-dialog(true)" );
+		shuffle_menu_item = g_menu_item_new(_("_Shuffle"), "win.shuffle-playlist");
+		loop_menu_item = g_menu_item_new(_("L_oop"), "win.toggle-loop");
 
 		g_menu_append_item(menu, add_menu_item);
 		g_menu_append_item(menu, add_loc_menu_item);
+		g_menu_append_item(menu, shuffle_menu_item);
 		g_menu_append_item(menu, loop_menu_item);
 		g_menu_freeze(menu);
 
@@ -733,11 +737,14 @@ void gmpv_playlist_widget_update_contents(	GmpvPlaylistWidget *wgt,
 
 	g_assert(playlist);
 
+	g_signal_handlers_block_by_func(wgt->store, row_inserted_handler, wgt);
+	g_signal_handlers_block_by_func(wgt->store, row_deleted_handler, wgt);
+
 	iter_end = !gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
 
 	for(guint i = 0; i < playlist->len; i++)
 	{
-		GmpvPlaylistEntry *entry = g_ptr_array_index(playlist, i);
+		GmpvPlaylistEntry *entry= g_ptr_array_index(playlist, i);
 		gchar *uri = entry->filename;
 		gchar *title = entry->title;
 		gchar *name = title?g_strdup(title):get_name_from_path(uri);
@@ -745,18 +752,19 @@ void gmpv_playlist_widget_update_contents(	GmpvPlaylistWidget *wgt,
 		/* Overwrite current entry if it doesn't match the new value */
 		if(!iter_end)
 		{
-			gchar *old_name = NULL;
-			gchar *old_uri = NULL;
-			gboolean name_update;
-			gboolean uri_update;
+			gchar *old_name =	NULL;
+			gchar *old_uri =	NULL;
+			gboolean name_update =	FALSE;
+			gboolean uri_update =	FALSE;
 
-			gtk_tree_model_get
-				(	GTK_TREE_MODEL(store), &iter,
-					PLAYLIST_NAME_COLUMN, &old_name,
-					PLAYLIST_URI_COLUMN, &old_uri, -1 );
+			gtk_tree_model_get(	GTK_TREE_MODEL(store),
+						&iter,
+						PLAYLIST_NAME_COLUMN, &old_name,
+						PLAYLIST_URI_COLUMN, &old_uri,
+						-1 );
 
-			name_update = (g_strcmp0(name, old_name) != 0);
-			uri_update = (g_strcmp0(uri, old_uri) != 0);
+			name_update =	(g_strcmp0(name, old_name) != 0);
+			uri_update =	(g_strcmp0(uri, old_uri) != 0);
 
 			/* Only set the name if either the title can be
 			 * retrieved or the name is unset. This preserves the
@@ -765,19 +773,23 @@ void gmpv_playlist_widget_update_contents(	GmpvPlaylistWidget *wgt,
 			 */
 			if(name_update && (!old_name || title || uri_update))
 			{
-				gtk_list_store_set
-					(	store, &iter,
-						PLAYLIST_NAME_COLUMN, name, -1 );
+				gtk_list_store_set(	store,
+							&iter,
+							PLAYLIST_NAME_COLUMN,
+							name,
+							-1 );
 			}
 
 			if(uri_update)
 			{
-				gtk_list_store_set
-					(	store, &iter,
-						PLAYLIST_URI_COLUMN, uri, -1 );
+				gtk_list_store_set(	store,
+							&iter,
+							PLAYLIST_URI_COLUMN,
+							uri,
+							-1 );
 			}
 
-			iter_end = !gtk_tree_model_iter_next
+			iter_end =	!gtk_tree_model_iter_next
 					(GTK_TREE_MODEL(store), &iter);
 
 			g_free(old_name);
@@ -789,12 +801,18 @@ void gmpv_playlist_widget_update_contents(	GmpvPlaylistWidget *wgt,
 		else
 		{
 			gtk_list_store_append(store, &iter);
-			gtk_list_store_set(	store, &iter,
-						PLAYLIST_NAME_COLUMN, name,
+			gtk_list_store_set(	store,
+						&iter,
+						PLAYLIST_NAME_COLUMN,
+						name,
 						-1 );
-			gtk_list_store_set(	store, &iter,
-						PLAYLIST_URI_COLUMN, uri,
+			gtk_list_store_set(	store,
+						&iter,
+						PLAYLIST_URI_COLUMN,
+						uri,
 						-1 );
+
+			wgt->playlist_count++;
 		}
 
 		g_free(name);
@@ -805,8 +823,15 @@ void gmpv_playlist_widget_update_contents(	GmpvPlaylistWidget *wgt,
 	 */
 	if(!iter_end)
 	{
-		while(gtk_list_store_remove(store, &iter));
+		while(gtk_list_store_remove(store, &iter))
+		{
+			wgt->playlist_count--;
+		}
 	}
+
+	g_signal_handlers_unblock_by_func(wgt->store, row_inserted_handler, wgt);
+	g_signal_handlers_unblock_by_func(wgt->store, row_deleted_handler, wgt);
+	g_object_notify(G_OBJECT(wgt), "playlist-count");
 }
 
 GPtrArray *gmpv_playlist_widget_get_contents(GmpvPlaylistWidget *wgt)
