@@ -65,9 +65,10 @@ static void get_property(	GObject *object,
 				guint property_id,
 				GValue *value,
 				GParamSpec *pspec );
+static void destroy(GtkWidget *widget);
 static void set_cursor_visible(GmpvVideoArea *area, gboolean visible);
 static gboolean timeout_handler(gpointer data);
-static gboolean motion_notify_handler(GtkWidget *widget, GdkEventMotion *event);
+static gboolean motion_notify_event(GtkWidget *widget, GdkEventMotion *event);
 static gboolean fs_control_crossing_handler(	GtkWidget *widget,
 						GdkEventCrossing *event,
 						gpointer data );
@@ -116,6 +117,17 @@ static void get_property(	GObject *object,
 	}
 }
 
+static void destroy(GtkWidget *widget)
+{
+	GmpvVideoArea *area = GMPV_VIDEO_AREA(widget);
+
+	if(area->timeout_tag > 0)
+	{
+		g_source_remove(area->timeout_tag);
+		area->timeout_tag = 0;
+	}
+}
+
 static void set_cursor_visible(GmpvVideoArea *area, gboolean visible)
 {
 	GdkWindow *window;
@@ -170,7 +182,7 @@ static gboolean timeout_handler(gpointer data)
 	return (area->timeout_tag != 0);
 }
 
-static gboolean motion_notify_handler(GtkWidget *widget, GdkEventMotion *event)
+static gboolean motion_notify_event(GtkWidget *widget, GdkEventMotion *event)
 {
 	GmpvVideoArea *area = GMPV_VIDEO_AREA(widget);
 	GdkCursor *cursor;
@@ -189,6 +201,7 @@ static gboolean motion_notify_handler(GtkWidget *widget, GdkEventMotion *event)
 	if(area->timeout_tag > 0)
 	{
 		g_source_remove(area->timeout_tag);
+		area->timeout_tag = 0;
 	}
 
 	area->timeout_tag = g_timeout_add_seconds(	FS_CONTROL_HIDE_DELAY,
@@ -205,7 +218,7 @@ static gboolean render_handler(	GtkGLArea *gl_area,
 {
 	g_signal_emit_by_name(data, "render");
 
-	return FALSE;
+	return TRUE;
 }
 
 static void notify_handler(	GObject *gobject,
@@ -226,7 +239,7 @@ static void notify_handler(	GObject *gobject,
 			"child-revealed", &child_revealed,
 			NULL );
 
-	gtk_widget_set_visible(	GTK_WIDGET(area),
+	gtk_widget_set_visible(	GTK_WIDGET(area->header_bar_revealer),
 				area->fullscreen &&
 				(reveal_child || child_revealed) );
 }
@@ -248,7 +261,8 @@ static void gmpv_video_area_class_init(GmpvVideoAreaClass *klass)
 
 	obj_class->set_property = set_property;
 	obj_class->get_property = get_property;
-	wgt_class->motion_notify_event = motion_notify_handler;
+	wgt_class->destroy = destroy;
+	wgt_class->motion_notify_event = motion_notify_event;
 
 	g_signal_new(	"render",
 			G_TYPE_FROM_CLASS(klass),
@@ -285,7 +299,7 @@ static void gmpv_video_area_init(GmpvVideoArea *area)
 	area->stack = gtk_stack_new();
 	area->draw_area = gtk_drawing_area_new();
 	area->gl_area = gtk_gl_area_new();
-	area->control_box = NULL;
+	area->control_box = gmpv_control_box_new();
 	area->header_bar = gmpv_header_bar_new();
 	area->control_box_revealer = gtk_revealer_new();
 	area->header_bar_revealer = gtk_revealer_new();
@@ -307,15 +321,17 @@ static void gmpv_video_area_init(GmpvVideoArea *area)
 	gtk_widget_add_events(area->gl_area, extra_events);
 
 	gtk_widget_set_valign(area->control_box_revealer, GTK_ALIGN_END);
-	gtk_widget_show(area->control_box_revealer);
 	gtk_revealer_set_reveal_child
 		(GTK_REVEALER(area->control_box_revealer), FALSE);
 
 	gtk_widget_set_valign(area->header_bar_revealer, GTK_ALIGN_START);
-	gtk_widget_show(area->header_bar_revealer);
 	gtk_revealer_set_reveal_child
 		(GTK_REVEALER(area->header_bar_revealer), FALSE);
 
+	gtk_widget_show_all(area->control_box);
+	gtk_widget_hide(area->control_box_revealer);
+	gtk_widget_set_no_show_all(area->control_box_revealer, TRUE);
+
 	gtk_widget_show_all(area->header_bar);
 	gtk_widget_hide(area->header_bar_revealer);
 	gtk_widget_set_no_show_all(area->header_bar_revealer, TRUE);
@@ -327,11 +343,11 @@ static void gmpv_video_area_init(GmpvVideoArea *area)
 	g_signal_connect(	area->header_bar_revealer,
 				"notify::reveal-child",
 				G_CALLBACK(notify_handler),
-				area->header_bar_revealer );
+				area );
 	g_signal_connect(	area->header_bar_revealer,
 				"notify::child-revealed",
 				G_CALLBACK(notify_handler),
-				area->header_bar_revealer );
+				area );
 	g_signal_connect(	area,
 				"enter-notify-event",
 				G_CALLBACK(fs_control_crossing_handler),
@@ -347,6 +363,8 @@ static void gmpv_video_area_init(GmpvVideoArea *area)
 
 	gtk_container_add(	GTK_CONTAINER(area->header_bar_revealer),
 				area->header_bar );
+	gtk_container_add(	GTK_CONTAINER(area->control_box_revealer),
+				area->control_box );
 
 	gtk_overlay_add_overlay(GTK_OVERLAY(area), area->control_box_revealer);
 	gtk_overlay_add_overlay(GTK_OVERLAY(area), area->header_bar_revealer);
@@ -391,22 +409,10 @@ void gmpv_video_area_set_fullscreen_state(	GmpvVideoArea *area,
 	}
 }
 
-void gmpv_video_area_set_control_box(	GmpvVideoArea *area,
-					GtkWidget *control_box )
+void gmpv_video_area_set_control_box_visible(	GmpvVideoArea *area,
+						gboolean visible )
 {
-	GtkContainer *revealer = GTK_CONTAINER(area->control_box_revealer);
-
-	if(area->control_box)
-	{
-		gtk_container_remove(revealer, area->control_box);
-	}
-
-	area->control_box = control_box;
-
-	if(control_box)
-	{
-		gtk_container_add(revealer, control_box);
-	}
+	gtk_widget_set_visible(area->control_box_revealer, visible);
 }
 
 void gmpv_video_area_set_use_opengl(GmpvVideoArea *area, gboolean use_opengl)
@@ -431,6 +437,11 @@ GtkGLArea *gmpv_video_area_get_gl_area(GmpvVideoArea *area)
 	return GTK_GL_AREA(area->gl_area);
 }
 
+GmpvControlBox *gmpv_video_area_get_control_box(GmpvVideoArea *area)
+{
+	return GMPV_CONTROL_BOX(area->control_box);
+}
+
 gint64 gmpv_video_area_get_xid(GmpvVideoArea *area)
 {
 #ifdef GDK_WINDOWING_X11
