@@ -22,10 +22,11 @@
 #include "gmpv_view.h"
 #include "gmpv_file_chooser.h"
 #include "gmpv_open_location_dialog.h"
-#include "gmpv_pref_dialog.h"
+#include "gmpv_preferences_dialog.h"
 #include "gmpv_shortcuts_window.h"
 #include "gmpv_authors.h"
 #include "gmpv_marshal.h"
+#include "gmpv_menu.h"
 #include "gmpv_common.h"
 #include "gmpv_def.h"
 
@@ -41,7 +42,6 @@ enum
 	PROP_PLAYLIST_POS,
 	PROP_TRACK_LIST,
 	PROP_CHAPTERS_ENABLED,
-	PROP_CONTROL_BOX_ENABLED,
 	PROP_FULLSCREEN,
 	N_PROPERTIES
 };
@@ -80,6 +80,8 @@ static void get_property(	GObject *object,
 				guint property_id,
 				GValue *value,
 				GParamSpec *pspec );
+static void dispose(GObject * object);
+static void finalize(GObject * object);
 static void load_css(GmpvView *view);
 static void save_playlist(GmpvView *view, GFile *file, GError **error);
 static void show_message_dialog(	GmpvMainWindow *wnd,
@@ -88,14 +90,11 @@ static void show_message_dialog(	GmpvMainWindow *wnd,
 					const gchar *prefix,
 					const gchar *msg );
 
-/* Property changes */
-static void play_button_handler(GtkButton *button, gpointer data);
-static void stop_button_handler(GtkButton *button, gpointer data);
-static void next_button_handler(GtkButton *button, gpointer data);
-static void previous_button_handler(GtkButton *button, gpointer data);
-static void forward_button_handler(GtkButton *button, gpointer data);
-static void rewind_button_handler(GtkButton *button, gpointer data);
-static void seek_handler(GtkWidget *widget, gdouble value,  gpointer data);
+/* Control box signals */
+static void button_clicked_handler(	GtkWidget *widget,
+					const gchar *button,
+					gpointer data );
+static void seek_handler(GtkWidget *widget, gdouble value, gpointer data);
 
 /* Dialog responses */
 static void open_dialog_response_handler(	GtkDialog *dialog,
@@ -185,13 +184,10 @@ static void constructed(GObject *object)
 	load_css(view);
 	gtk_widget_show_all(GTK_WIDGET(view->wnd));
 
-	if(csd_enable)
-	{
-		gmpv_control_box_set_fullscreen_button_visible(control_box, FALSE);
-	}
-
-	gmpv_control_box_set_chapter_enabled(control_box, FALSE);
-
+	g_object_set(	control_box,
+			"show-fullscreen-button", !csd_enable,
+			"chapters-enabled", FALSE,
+			NULL );
 	g_object_set(	gtk_settings_get_default(),
 			"gtk-application-prefer-dark-theme",
 			dark_theme_enable,
@@ -202,6 +198,15 @@ static void constructed(GObject *object)
 	g_object_bind_property(	view, "title",
 				view->wnd, "title",
 				G_BINDING_DEFAULT );
+	g_object_bind_property(	view, "duration",
+				control_box, "duration",
+				G_BINDING_DEFAULT );
+	g_object_bind_property(	view, "pause",
+				control_box, "pause",
+				G_BINDING_DEFAULT );
+	g_object_bind_property(	view, "chapters-enabled",
+				control_box, "chapters-enabled",
+				G_BINDING_DEFAULT );
 	g_object_bind_property(	view, "volume",
 				control_box, "volume",
 				G_BINDING_BIDIRECTIONAL );
@@ -209,31 +214,11 @@ static void constructed(GObject *object)
 				view, "playlist-count",
 				G_BINDING_DEFAULT );
 
-	g_signal_connect(	control_box,
-				"play-button-clicked",
-				G_CALLBACK(play_button_handler),
+	g_signal_connect(	view->wnd,
+				"button-clicked",
+				G_CALLBACK(button_clicked_handler),
 				view );
-	g_signal_connect(	control_box,
-				"stop-button-clicked",
-				G_CALLBACK(stop_button_handler),
-				view );
-	g_signal_connect(	control_box,
-				"next-button-clicked",
-				G_CALLBACK(next_button_handler),
-				view );
-	g_signal_connect(	control_box,
-				"previous-button-clicked",
-				G_CALLBACK(previous_button_handler),
-				view );
-	g_signal_connect(	control_box,
-				"forward-button-clicked",
-				G_CALLBACK(forward_button_handler),
-				view );
-	g_signal_connect(	control_box,
-				"rewind-button-clicked",
-				G_CALLBACK(rewind_button_handler),
-				view );
-	g_signal_connect(	control_box,
+	g_signal_connect(	view->wnd,
 				"seek",
 				G_CALLBACK(seek_handler),
 				view );
@@ -307,6 +292,8 @@ static void constructed(GObject *object)
 				"rows-reordered",
 				G_CALLBACK(playlist_row_reordered_handler),
 				view );
+
+	G_OBJECT_CLASS(gmpv_view_parent_class)->constructed(object);
 }
 
 static void set_property(	GObject *object,
@@ -330,8 +317,11 @@ static void set_property(	GObject *object,
 
 		case PROP_PLAYLIST_COUNT:
 		self->playlist_count = g_value_get_int(value);
-		gmpv_control_box_set_enabled
-			(control_box, self->playlist_count > 0);
+
+		g_object_set(	control_box,
+				"enabled",
+				self->playlist_count > 0,
+				NULL );
 
 		if(self->playlist_count <= 0)
 		{
@@ -341,7 +331,6 @@ static void set_property(	GObject *object,
 
 		case PROP_PAUSE:
 		self->pause = g_value_get_boolean(value);
-		gmpv_control_box_set_playing_state(control_box, !self->pause);
 		break;
 
 		case PROP_TITLE:
@@ -355,8 +344,6 @@ static void set_property(	GObject *object,
 
 		case PROP_DURATION:
 		self->duration = g_value_get_double(value);
-		gmpv_control_box_set_seek_bar_duration
-			(control_box, (gint)self->duration);
 		break;
 
 		case PROP_PLAYLIST_POS:
@@ -372,14 +359,6 @@ static void set_property(	GObject *object,
 
 		case PROP_CHAPTERS_ENABLED:
 		self->chapters_enabled = g_value_get_boolean(value);
-		gmpv_control_box_set_chapter_enabled
-			(control_box, self->chapters_enabled);
-		break;
-
-		case PROP_CONTROL_BOX_ENABLED:
-		self->control_box_enabled = g_value_get_boolean(value);
-		gmpv_control_box_set_enabled
-			(control_box, self->control_box_enabled);
 		break;
 
 		case PROP_FULLSCREEN:
@@ -438,10 +417,6 @@ static void get_property(	GObject *object,
 		g_value_set_boolean(value, self->chapters_enabled);
 		break;
 
-		case PROP_CONTROL_BOX_ENABLED:
-		g_value_set_boolean(value, self->control_box_enabled);
-		break;
-
 		case PROP_FULLSCREEN:
 		g_value_set_boolean(value, self->fullscreen);
 		break;
@@ -452,6 +427,26 @@ static void get_property(	GObject *object,
 	}
 }
 
+static void dispose(GObject *object)
+{
+	GmpvView *view = GMPV_VIEW(object);
+
+	if(view->wnd)
+	{
+		gtk_widget_destroy(GTK_WIDGET(view->wnd));
+		view->wnd = NULL;
+	}
+
+	G_OBJECT_CLASS(gmpv_view_parent_class)->dispose(object);
+}
+
+static void finalize(GObject *object)
+{
+	g_free(GMPV_VIEW(object)->title);
+
+	G_OBJECT_CLASS(gmpv_view_parent_class)->finalize(object);
+}
+
 static void load_css(GmpvView *view)
 {
 	const gchar *style;
@@ -582,34 +577,14 @@ void show_message_dialog(	GmpvMainWindow *wnd,
 	gtk_widget_show_all(dialog);
 }
 
-static void play_button_handler(GtkButton *button, gpointer data)
+static void button_clicked_handler(	GtkWidget *widget,
+					const gchar *button,
+					gpointer data )
 {
-	g_signal_emit_by_name(data, "button-clicked::play");
-}
+	gchar *name = g_strconcat("button-clicked::", button, NULL);
 
-static void stop_button_handler(GtkButton *button, gpointer data)
-{
-	g_signal_emit_by_name(data, "button-clicked::stop");
-}
-
-static void next_button_handler(GtkButton *button, gpointer data)
-{
-	g_signal_emit_by_name(data, "button-clicked::next");
-}
-
-static void previous_button_handler(GtkButton *button, gpointer data)
-{
-	g_signal_emit_by_name(data, "button-clicked::previous");
-}
-
-static void forward_button_handler(GtkButton *button, gpointer data)
-{
-	g_signal_emit_by_name(data, "button-clicked::forward");
-}
-
-static void rewind_button_handler(GtkButton *button, gpointer data)
-{
-	g_signal_emit_by_name(data, "button-clicked::rewind");
+	g_signal_emit_by_name(data, name);
+	g_free(name);
 }
 
 static void seek_handler(GtkWidget *widget, gdouble value, gpointer data)
@@ -722,16 +697,10 @@ static void preferences_dialog_response_handler(	GtkDialog *dialog,
 		GmpvView *view;
 		GSettings *settings;
 		gboolean csd_enable;
-		gboolean dark_theme_enable;
-		gboolean always_floating;
 
 		view = data;
 		settings = g_settings_new(CONFIG_ROOT);
 		csd_enable = g_settings_get_boolean(settings, "csd-enable");
-		dark_theme_enable =	g_settings_get_boolean
-					(settings, "dark-theme-enable");
-		always_floating =	g_settings_get_boolean
-					(settings, "always-use-floating-controls");
 
 		if(gmpv_main_window_get_csd_enabled(view->wnd) != csd_enable)
 		{
@@ -745,15 +714,6 @@ static void preferences_dialog_response_handler(	GtkDialog *dialog,
 						"take effect.") );
 		}
 
-		g_object_set(	gtk_settings_get_default(),
-				"gtk-application-prefer-dark-theme",
-				dark_theme_enable,
-				NULL );
-		g_object_set(	view->wnd,
-				"always-use-floating-controls",
-				always_floating,
-				NULL );
-
 		gtk_widget_queue_draw(GTK_WIDGET(view->wnd));
 		g_signal_emit_by_name(data, "preferences-updated");
 
@@ -830,8 +790,7 @@ static gboolean draw_handler(GtkWidget *widget, cairo_t *cr, gpointer data)
 		GmpvControlBox *control_box;
 
 		control_box = gmpv_main_window_get_control_box(view->wnd);
-
-		gmpv_control_box_set_enabled(control_box, FALSE);
+		g_object_set(control_box, "enabled", FALSE, NULL);
 	}
 
 	g_signal_emit_by_name(view, "ready");
@@ -961,6 +920,8 @@ static void gmpv_view_class_init(GmpvViewClass *klass)
 	object_class->constructed = constructed;
 	object_class->set_property = set_property;
 	object_class->get_property = get_property;
+	object_class->dispose = dispose;
+	object_class->finalize = finalize;
 
 	pspec = g_param_spec_pointer
 		(	"window",
@@ -1274,10 +1235,33 @@ static void gmpv_view_init(GmpvView *view)
 	view->fullscreen = FALSE;
 }
 
-GmpvView *gmpv_view_new(GmpvMainWindow *wnd)
+GmpvView *gmpv_view_new(GmpvApplication *app, gboolean always_floating)
 {
+	GtkApplication *gtk_app = GTK_APPLICATION(app);
+	GSettings *settings = g_settings_new(CONFIG_ROOT);
+	GtkWidget *window = gmpv_main_window_new(gtk_app, always_floating);
+
+	if(g_settings_get_boolean(settings, "csd-enable"))
+	{
+		GMenu *app_menu = g_menu_new();
+
+		gmpv_menu_build_app_menu(app_menu);
+		gtk_application_set_app_menu(gtk_app, G_MENU_MODEL(app_menu));
+		gmpv_main_window_enable_csd(GMPV_MAIN_WINDOW(window));
+	}
+	else
+	{
+		GMenu *full_menu = g_menu_new();
+
+		gmpv_menu_build_full(full_menu, NULL);
+		gtk_application_set_app_menu(gtk_app, NULL);
+		gtk_application_set_menubar(gtk_app, G_MENU_MODEL(full_menu));
+	}
+
+	g_object_unref(settings);
+
 	return GMPV_VIEW(g_object_new(	gmpv_view_get_type(),
-					"window", wnd,
+					"window", window,
 					NULL ));
 }
 
@@ -1423,14 +1407,14 @@ void gmpv_view_show_save_playlist_dialog(GmpvView *view)
 
 void gmpv_view_show_preferences_dialog(GmpvView *view)
 {
-	GtkWidget *pref_dialog = gmpv_pref_dialog_new(GTK_WINDOW(view->wnd));
+	GtkWidget *dialog = gmpv_preferences_dialog_new(GTK_WINDOW(view->wnd));
 
-	g_signal_connect_after(	pref_dialog,
+	g_signal_connect_after(	dialog,
 				"response",
 				G_CALLBACK(preferences_dialog_response_handler),
 				view );
 
-	gtk_widget_show_all(pref_dialog);
+	gtk_widget_show_all(dialog);
 }
 
 void gmpv_view_show_shortcuts_dialog(GmpvView *view)
@@ -1531,6 +1515,46 @@ void gmpv_view_get_video_area_geometry(GmpvView *view, gint *width, gint *height
 	*height = allocation.height;
 }
 
+void gmpv_view_move(	GmpvView *view,
+			gboolean flip_x,
+			gboolean flip_y,
+			GValue *x,
+			GValue *y )
+{
+	GdkScreen *screen = gdk_screen_get_default();
+	GtkWindow *window = GTK_WINDOW(view->wnd);
+	gint64 x_pos = -1;
+	gint64 y_pos = -1;
+	gint window_width = 0;
+	gint window_height = 0;
+	gint space_x;
+	gint space_y;
+
+	gtk_window_get_size(window, &window_width, &window_height);
+	space_x = gdk_screen_get_width(screen)-window_width;
+	space_y = gdk_screen_get_height(screen)-window_height;
+
+	if(G_VALUE_HOLDS_DOUBLE(x))
+	{
+		x_pos = (gint64)(g_value_get_double(x)*space_x);
+	}
+	else
+	{
+		x_pos = flip_x?space_x-g_value_get_int64(x):g_value_get_int64(x);
+	}
+
+	if(G_VALUE_HOLDS_DOUBLE(y))
+	{
+		y_pos = (gint64)(g_value_get_double(y)*space_y);
+	}
+	else
+	{
+		y_pos = flip_y?space_y-g_value_get_int64(y):g_value_get_int64(y);
+	}
+
+	gtk_window_move(window, (gint)x_pos, (gint)y_pos);
+}
+
 void gmpv_view_resize_video_area(GmpvView *view, gint width, gint height)
 {
 	gmpv_main_window_resize_video_area(view->wnd, width, height);
@@ -1547,20 +1571,38 @@ void gmpv_view_set_time_position(GmpvView *view, gdouble position)
 	GmpvControlBox *control_box;
 
 	control_box = gmpv_main_window_get_control_box(view->wnd);
-	gmpv_control_box_set_seek_bar_pos(control_box, position);
+	g_object_set(control_box, "time-position", position, NULL);
 }
 
 void gmpv_view_update_playlist(GmpvView *view, GPtrArray *playlist)
 {
-	if(playlist)
-	{
-		GmpvPlaylistWidget *pl = gmpv_main_window_get_playlist(view->wnd);
+	GmpvPlaylistWidget *wgt = gmpv_main_window_get_playlist(view->wnd);
 
-		gmpv_playlist_widget_update_contents(pl, playlist);
-	}
+	gmpv_playlist_widget_update_contents(wgt, playlist);
 }
 
 void gmpv_view_set_playlist_pos(GmpvView *view, gint64 pos)
 {
 	g_object_set(view, "playlist-pos", pos, NULL);
 }
+
+void gmpv_view_set_playlist_visible(GmpvView *view, gboolean visible)
+{
+	gmpv_main_window_set_playlist_visible(view->wnd, visible);
+}
+
+gboolean gmpv_view_get_playlist_visible(GmpvView *view)
+{
+	return gmpv_main_window_get_playlist_visible(view->wnd);
+}
+
+void gmpv_view_set_controls_visible(GmpvView *view, gboolean visible)
+{
+	gmpv_main_window_set_controls_visible(view->wnd, visible);
+}
+
+gboolean gmpv_view_get_controls_visible(GmpvView *view)
+{
+	return gmpv_main_window_get_controls_visible(view->wnd);
+}
+
