@@ -24,7 +24,6 @@
 #include <string.h>
 
 #include "gmpv_mpris_player.h"
-#include "gmpv_application_private.h"
 #include "gmpv_common.h"
 #include "gmpv_main_window.h"
 #include "gmpv_mpris.h"
@@ -35,14 +34,14 @@
 enum
 {
 	PROP_0,
-	PROP_APP,
+	PROP_CONTROLLER,
 	N_PROPERTIES
 };
 
 struct _GmpvMprisPlayer
 {
 	GmpvMprisModule parent;
-	GmpvApplication *app;
+	GmpvController *controller;
 	guint reg_id;
 };
 
@@ -86,8 +85,12 @@ static gboolean set_prop_handler(	GDBusConnection *connection,
 					GVariant *value,
 					GError **error,
 					gpointer data );
+
 static void update_playback_status(GmpvMprisPlayer *player);
 static void update_playlist_state(GmpvMprisPlayer *player);
+static void update_speed(GmpvMprisPlayer *player);
+static void update_metadata(GmpvMprisPlayer *player);
+static void update_volume(GmpvMprisPlayer *player);
 
 static void idle_active_handler(	GObject *object,
 					GParamSpec *pspec,
@@ -120,7 +123,7 @@ G_DEFINE_TYPE(GmpvMprisPlayer, gmpv_mpris_player, GMPV_TYPE_MPRIS_MODULE);
 static void register_interface(GmpvMprisModule *module)
 {
 	GmpvMprisPlayer *player = GMPV_MPRIS_PLAYER(module);
-	GmpvModel *model = player->app->model;
+	GmpvModel *model = gmpv_controller_get_model(player->controller);
 	GDBusConnection *conn;
 	GDBusInterfaceInfo *iface;
 	GDBusInterfaceVTable vtable;
@@ -204,6 +207,12 @@ static void register_interface(GmpvMprisModule *module)
 					player,
 					NULL,
 					NULL );
+
+	update_playback_status(player);
+	update_playlist_state(player);
+	update_speed(player);
+	update_metadata(player);
+	update_volume(player);
 }
 
 static void unregister_interface(GmpvMprisModule *module)
@@ -224,8 +233,8 @@ static void set_property(	GObject *object,
 
 	switch(property_id)
 	{
-		case PROP_APP:
-		self->app = g_value_get_pointer(value);
+		case PROP_CONTROLLER:
+		self->controller = g_value_get_pointer(value);
 		break;
 
 		default:
@@ -243,8 +252,8 @@ static void get_property(	GObject *object,
 
 	switch(property_id)
 	{
-		case PROP_APP:
-		g_value_set_pointer(value, self->app);
+		case PROP_CONTROLLER:
+		g_value_set_pointer(value, self->controller);
 		break;
 
 		default:
@@ -288,11 +297,7 @@ static void append_metadata_tags(GVariantBuilder *builder, GPtrArray *list)
 		tag_name = tag_map[j].mpv_name?tag_map[j].tag_name:entry->key;
 		is_array = tag_map[j].mpv_name?tag_map[j].is_array:FALSE;
 
-		if(!is_array)
-		{
-			tag_value = g_variant_new_string(entry->value);
-		}
-		else if(is_array)
+		if(is_array)
 		{
 			GVariantBuilder tag_builder;
 			GVariant *elem_value;
@@ -306,6 +311,10 @@ static void append_metadata_tags(GVariantBuilder *builder, GPtrArray *list)
 
 			tag_value = g_variant_new("as", &tag_builder);
 		}
+		else
+		{
+			tag_value = g_variant_new_string(entry->value);
+		}
 
 		g_debug("Adding metadata tag \"%s\"", tag_name);
 		g_variant_builder_add(builder, "{sv}", tag_name, tag_value);
@@ -322,7 +331,7 @@ static void method_handler(	GDBusConnection *connection,
 				gpointer data )
 {
 	GmpvMprisPlayer *player = data;
-	GmpvModel *model = player->app->model;
+	GmpvModel *model = gmpv_controller_get_model(player->controller);
 
 	if(g_strcmp0(method_name, "Next") == 0)
 	{
@@ -404,7 +413,7 @@ static GVariant *get_prop_handler(	GDBusConnection *connection,
 		GmpvModel *model;
 		gdouble position;
 
-		model = player->app->model;
+		model = gmpv_controller_get_model(player->controller);
 		position = gmpv_model_get_time_position(model);
 		value = g_variant_new_int64((gint64)(position*1e6));
 	}
@@ -428,7 +437,7 @@ static gboolean set_prop_handler(	GDBusConnection *connection,
 					gpointer data )
 {
 	GmpvMprisPlayer *player = data;
-	GmpvModel *model = player->app->model;
+	GmpvModel *model = gmpv_controller_get_model(player->controller);
 
 	if(g_strcmp0(property_name, "Rate") == 0)
 	{
@@ -452,7 +461,7 @@ static gboolean set_prop_handler(	GDBusConnection *connection,
 
 static void update_playback_status(GmpvMprisPlayer *player)
 {
-	GmpvModel *model = player->app->model;
+	GmpvModel *model = gmpv_controller_get_model(player->controller);
 	const gchar *state;
 	gint idle_active;
 	gint core_idle;
@@ -489,7 +498,7 @@ static void update_playback_status(GmpvMprisPlayer *player)
 
 static void update_playlist_state(GmpvMprisPlayer *player)
 {
-	GmpvModel *model = player->app->model;
+	GmpvModel *model = gmpv_controller_get_model(player->controller);
 	gboolean can_prev;
 	gboolean can_next;
 	gint64 playlist_count;
@@ -512,53 +521,22 @@ static void update_playlist_state(GmpvMprisPlayer *player)
 						NULL );
 }
 
-static void idle_active_handler(	GObject *object,
-					GParamSpec *pspec,
-					gpointer data )
-{
-	update_playback_status(data);
-}
-
-static void core_idle_handler(	GObject *object,
-				GParamSpec *pspec,
-				gpointer data )
-{
-	update_playback_status(data);
-}
-
-static void playlist_pos_handler(	GObject *object,
-					GParamSpec *pspec,
-					gpointer data )
-{
-	update_playlist_state(data);
-}
-
-static void playlist_count_handler(	GObject *object,
-					GParamSpec *pspec,
-					gpointer data )
-{
-	update_playlist_state(data);
-}
-
-static void speed_handler(	GObject *object,
-				GParamSpec *pspec,
-				gpointer data )
+static void update_speed(GmpvMprisPlayer *player)
 {
+	GmpvModel *model = gmpv_controller_get_model(player->controller);
 	gdouble speed = 1.0;
 
-	g_object_get(object, "speed", &speed, NULL);
+	g_object_get(G_OBJECT(model), "speed", &speed, NULL);
 
-	gmpv_mpris_module_set_properties(	GMPV_MPRIS_MODULE(data),
+	gmpv_mpris_module_set_properties(	GMPV_MPRIS_MODULE(player),
 						"Rate",
 						g_variant_new_double(speed),
 						NULL );
 }
 
-static void metadata_handler(	GObject *object,
-				GParamSpec *pspec,
-				gpointer data )
+static void update_metadata(GmpvMprisPlayer *player)
 {
-	GmpvModel *model = GMPV_MODEL(object);
+	GmpvModel *model = gmpv_controller_get_model(player->controller);
 	GPtrArray *metadata = NULL;
 	GVariantBuilder builder;
 	gchar *path;
@@ -569,7 +547,7 @@ static void metadata_handler(	GObject *object,
 	gint64 playlist_pos = 0;
 
 	g_variant_builder_init(&builder, G_VARIANT_TYPE("a{sv}"));
-	path = gmpv_model_get_current_path(model);
+	path = gmpv_model_get_current_path(model)?:g_strdup("");
 	uri = g_filename_to_uri(path, NULL, NULL)?:g_strdup(path);
 
 	if(uri)
@@ -604,7 +582,7 @@ static void metadata_handler(	GObject *object,
 
 	append_metadata_tags(&builder, metadata);
 
-	gmpv_mpris_module_set_properties(	GMPV_MPRIS_MODULE(data),
+	gmpv_mpris_module_set_properties(	GMPV_MPRIS_MODULE(player),
 						"Metadata",
 						g_variant_new("a{sv}", &builder),
 						NULL );
@@ -615,19 +593,67 @@ static void metadata_handler(	GObject *object,
 	g_free(trackid);
 }
 
+static void update_volume(GmpvMprisPlayer *player)
+{
+	GmpvModel *model = gmpv_controller_get_model(player->controller);
+	gdouble volume = 0.0;
+
+	g_object_get(G_OBJECT(model), "volume", &volume, NULL);
+
+	gmpv_mpris_module_set_properties
+		(	GMPV_MPRIS_MODULE(player),
+			"Volume",
+			g_variant_new_double(volume/100.0),
+			NULL );
+}
+
+static void idle_active_handler(	GObject *object,
+					GParamSpec *pspec,
+					gpointer data )
+{
+	update_playback_status(data);
+}
+
+static void core_idle_handler(	GObject *object,
+				GParamSpec *pspec,
+				gpointer data )
+{
+	update_playback_status(data);
+}
+
+static void playlist_pos_handler(	GObject *object,
+					GParamSpec *pspec,
+					gpointer data )
+{
+	update_playlist_state(data);
+}
+
+static void playlist_count_handler(	GObject *object,
+					GParamSpec *pspec,
+					gpointer data )
+{
+	update_playlist_state(data);
+}
+
+static void speed_handler(	GObject *object,
+				GParamSpec *pspec,
+				gpointer data )
+{
+	update_speed(data);
+}
+
+static void metadata_handler(	GObject *object,
+				GParamSpec *pspec,
+				gpointer data )
+{
+	update_metadata(data);
+}
+
 static void volume_handler(	GObject *object,
 				GParamSpec *pspec,
 				gpointer data )
 {
-	gdouble volume = 0.0;
-
-	g_object_get(object, "volume", &volume, NULL);
-
-	gmpv_mpris_module_set_properties
-		(	GMPV_MPRIS_MODULE(data),
-			"Volume",
-			g_variant_new_double(volume/100.0),
-			NULL );
+	update_volume(data);
 }
 
 static void playback_restart_handler(GmpvModel *model, gpointer data)
@@ -665,28 +691,28 @@ static void gmpv_mpris_player_class_init(GmpvMprisPlayerClass *klass)
 	object_class->get_property = get_property;
 
 	pspec = g_param_spec_pointer
-		(	"app",
-			"Application",
+		(	"controller",
+			"Controller",
 			"The GmpvApplication to use",
 			G_PARAM_CONSTRUCT_ONLY|G_PARAM_READWRITE );
-	g_object_class_install_property(object_class, PROP_APP, pspec);
+	g_object_class_install_property(object_class, PROP_CONTROLLER, pspec);
 }
 
 static void gmpv_mpris_player_init(GmpvMprisPlayer *player)
 {
-	player->app = NULL;
+	player->controller = NULL;
 	player->reg_id = 0;
 }
 
-GmpvMprisPlayer *gmpv_mpris_player_new(	GmpvApplication *app,
+GmpvMprisModule *gmpv_mpris_player_new(	GmpvController *controller,
 					GDBusConnection *conn )
 {
 	GDBusInterfaceInfo *iface;
 
 	iface = gmpv_mpris_org_mpris_media_player2_player_interface_info();
 
-	return GMPV_MPRIS_PLAYER(g_object_new(	gmpv_mpris_player_get_type(),
-						"app", app,
+	return GMPV_MPRIS_MODULE(g_object_new(	gmpv_mpris_player_get_type(),
+						"controller", controller,
 						"conn", conn,
 						"iface", iface,
 						NULL ));
